Package: gtools
Title: Various R Programming Tools
Description: Functions to assist in R programming, including:
  - assist in developing, updating, and maintaining R and R packages ('ask', 'checkRVersion',
    'getDependencies', 'keywords', 'scat'),
  - calculate the logit and inverse logit transformations ('logit', 'inv.logit'),
  - test if a value is missing, empty or contains only NA and NULL values ('invalid'),
  - manipulate R's .Last function ('addLast'),
  - define macros ('defmacro'),
  - detect odd and even integers ('odd', 'even'),
  - convert strings containing non-ASCII characters (like single quotes) to plain ASCII ('ASCIIfy'),
  - perform a binary search ('binsearch'),
  - sort strings containing both numeric and character components ('mixedsort'),
  - create a factor variable from the quantiles of a continuous variable ('quantcut'),
  - enumerate permutations and combinations ('combinations', 'permutation'),
  - calculate and convert between fold-change and log-ratio ('foldchange',
    'logratio2foldchange', 'foldchange2logratio'),
  - calculate probabilities and generate random numbers from Dirichlet distributions
    ('rdirichlet', 'ddirichlet'),
  - apply a function over adjacent subsets of a vector ('running'),
  - modify the TCP_NODELAY ('de-Nagle') flag for socket objects,
  - efficient 'rbind' of data frames, even if the column names don't match ('smartbind'),
  - generate significance stars from p-values ('stars.pval'),
  - convert characters to/from ASCII codes ('asc', 'chr'),
  - convert character vector to ASCII representation ('ASCIIfy'),
  - apply title capitalization rules to a character vector ('capwords').
Version: 3.9.5
Authors@R: c(person("Gregory R.", "Warnes", role = "aut"),
             person("Ben", "Bolker", role = c("aut", "cre"),
                     email = "bolker@mcmaster.ca",
                     comment=c(ORCID="0000-0002-2127-0443")),
             person("Thomas", "Lumley", role = "aut"),
             person("Arni", "Magnusson", role = "aut"),
             person("Bill", "Venables", role = "aut"),
             person("Genei", "Ryodan", role = "aut"),
             person("Steffen", "Moeller", role = "aut"),
             person("Ian", "Wilson", role = "ctb"),
             person("Mark", "Davis", role = "ctb"),
             person("Nitin", "Jain", role="ctb"),
             person("Scott", "Chamberlain", role = "ctb"))
License: GPL-2
Depends: methods, stats, utils
URL: https://github.com/r-gregmisc/gtools
BugReports: https://github.com/r-gregmisc/gtools/issues
Language: en-US
Suggests: car, gplots, knitr, rstudioapi, SGP, taxize
RoxygenNote: 7.2.3
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2023-11-19 00:52:21 UTC; bolker
Author: Gregory R. Warnes [aut],
  Ben Bolker [aut, cre] (<https://orcid.org/0000-0002-2127-0443>),
  Thomas Lumley [aut],
  Arni Magnusson [aut],
  Bill Venables [aut],
  Genei Ryodan [aut],
  Steffen Moeller [aut],
  Ian Wilson [ctb],
  Mark Davis [ctb],
  Nitin Jain [ctb],
  Scott Chamberlain [ctb]
Maintainer: Ben Bolker <bolker@mcmaster.ca>
Repository: CRAN
Date/Publication: 2023-11-20 15:10:05 UTC
Built: R 4.4.0; amd64-portbld-midnightbsd4; 2026-07-18 01:12:24 UTC; unix

Package: gtools
Title: Various R Programming Tools
Description: Functions to assist in R programming, including:
  - assist in developing, updating, and maintaining R and R packages ('ask', 'checkRVersion',
    'getDependencies', 'keywords', 'scat'),
  - calculate the logit and inverse logit transformations ('logit', 'inv.logit'),
  - test if a value is missing, empty or contains only NA and NULL values ('invalid'),
  - manipulate R's .Last function ('addLast'),
  - define macros ('defmacro'),
  - detect odd and even integers ('odd', 'even'),
  - convert strings containing non-ASCII characters (like single quotes) to plain ASCII ('ASCIIfy'),
  - perform a binary search ('binsearch'),
  - sort strings containing both numeric and character components ('mixedsort'),
  - create a factor variable from the quantiles of a continuous variable ('quantcut'),
  - enumerate permutations and combinations ('combinations', 'permutation'),
  - calculate and convert between fold-change and log-ratio ('foldchange',
    'logratio2foldchange', 'foldchange2logratio'),
  - calculate probabilities and generate random numbers from Dirichlet distributions
    ('rdirichlet', 'ddirichlet'),
  - apply a function over adjacent subsets of a vector ('running'),
  - modify the TCP_NODELAY ('de-Nagle') flag for socket objects,
  - efficient 'rbind' of data frames, even if the column names don't match ('smartbind'),
  - generate significance stars from p-values ('stars.pval'),
  - convert characters to/from ASCII codes ('asc', 'chr'),
  - convert character vector to ASCII representation ('ASCIIfy'),
  - apply title capitalization rules to a character vector ('capwords').
Version: 3.9.5
Authors@R: c(person("Gregory R.", "Warnes", role = "aut"),
             person("Ben", "Bolker", role = c("aut", "cre"),
                     email = "bolker@mcmaster.ca",
                     comment=c(ORCID="0000-0002-2127-0443")),
             person("Thomas", "Lumley", role = "aut"),
             person("Arni", "Magnusson", role = "aut"),
             person("Bill", "Venables", role = "aut"),
             person("Genei", "Ryodan", role = "aut"),
             person("Steffen", "Moeller", role = "aut"),
             person("Ian", "Wilson", role = "ctb"),
             person("Mark", "Davis", role = "ctb"),
             person("Nitin", "Jain", role="ctb"),
             person("Scott", "Chamberlain", role = "ctb"))
License: GPL-2
Depends: methods, stats, utils
URL: https://github.com/r-gregmisc/gtools
BugReports: https://github.com/r-gregmisc/gtools/issues
Language: en-US
Suggests: car, gplots, knitr, rstudioapi, SGP, taxize
RoxygenNote: 7.2.3
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2023-11-19 00:52:21 UTC; bolker
Author: Gregory R. Warnes [aut],
  Ben Bolker [aut, cre] (<https://orcid.org/0000-0002-2127-0443>),
  Thomas Lumley [aut],
  Arni Magnusson [aut],
  Bill Venables [aut],
  Genei Ryodan [aut],
  Steffen Moeller [aut],
  Ian Wilson [ctb],
  Mark Davis [ctb],
  Nitin Jain [ctb],
  Scott Chamberlain [ctb]
Maintainer: Ben Bolker <bolker@mcmaster.ca>
Repository: CRAN
Date/Publication: 2023-11-20 15:10:05 UTC

# Generated by roxygen2: do not edit by hand

export(ASCIIfy)
export(asc)
export(ask)
export(assert)
export(assignEdgewise)
export(baseOf)
export(binsearch)
export(capture)
export(capwords)
export(checkRVersion)
export(chr)
export(combinations)
export(ddirichlet)
export(defmacro)
export(even)
export(foldchange)
export(foldchange2logratio)
export(getDependencies)
export(inv.logit)
export(invalid)
export(keywords)
export(lastAdd)
export(loadedPackages)
export(logit)
export(logratio2foldchange)
export(mixedorder)
export(mixedsort)
export(na.replace)
export(odd)
export(permutations)
export(permute)
export(quantcut)
export(rdirichlet)
export(roman2int)
export(running)
export(scat)
export(script_file)
export(script_path)
export(setTCPNoDelay)
export(smartbind)
export(split_path)
export(sprint)
export(stars.pval)
export(stat_mode)
export(strmacro)
export(unByteCode)
export(unByteCodeAssign)
importFrom(methods,is)
importFrom(methods,new)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,rgamma)
importFrom(stats,symnum)
importFrom(utils,available.packages)
importFrom(utils,flush.console)
importFrom(utils,head)
importFrom(utils,help.search)
importFrom(utils,installed.packages)
importFrom(utils,modifyList)
importFrom(utils,packageVersion)
useDynLib(gtools, .registration=TRUE)

# gtools R package

<!-- badges: start -->

[![Project Status: Active - The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![R-CMD-check](https://github.com/r-gregmisc/gtools/workflows/R-CMD-check/badge.svg)](https://github.com/r-gregmisc/gtools/actions)
[![](https://www.r-pkg.org/badges/version/gtools)](https://www.r-pkg.org/pkg/gtools)
[![CRAN RStudio mirror
downloads](https://cranlogs.r-pkg.org/badges/gtools)](https://www.r-pkg.org/pkg/gtools)

<!-- badges: end -->

The `gtools` R package provides functions to assist in R programming,
including:

-   assist in developing, updating, and maintaining R and R packages
    (`ask`, `checkRVersion`, `getDependencies`, `keywords`, `scat`),
-   calculate the logit and inverse logit transformations (`logit`,
    `inv.logit`),
-   test if a value is missing, empty or contains only NA and NULL
    values (`invalid`),
-   manipulate R’s .Last function (`addLast`),
-   define macros (`defmacro`),
-   detect odd and even integers (`odd`, `even`),
-   convert strings containing non-ASCII characters (like single quotes)
    to plain ASCII (`ASCIIfy`),
-   perform a binary search (`binsearch`),
-   sort strings containing both numeric and character components
    (`mixedsort`),
-   create a factor variable from the quantiles of a continuous variable
    (`quantcut`),
-   enumerate permutations and combinations (`combinations`,
    `permutation`),
-   calculate and convert between fold-change and log-ratio
    (`foldchange`, `logratio2foldchange`, `foldchange2logratio`),
-   calculate probabilities and generate random numbers from Dirichlet
    distributions (`rdirichlet`, `ddirichlet`),
-   apply a function over adjacent subsets of a vector (`running`),
-   modify the TCP\_NODELAY (`de-Nagle`) flag for socket objects,
-   efficient `rbind` of data frames, even if the column names
    don`t match (`smartbind\`),
-   generate significance stars from p-values (`stars.pval`),
-   convert characters to/from ASCII codes (`asc`, `chr`),
-   convert character vector to ASCII representation (`ASCIIfy`).
-   apply title capitalization rules to a character vector (`capwords`)

## Installation

You can install the released version of gtools from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("gtools")
```

And the development version from [GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("r-gregmisc/gtools")
```

library(gtools)

# For debugging conveneince
cmdArgs <- commandArgs(trailingOnly = FALSE)
print(cmdArgs)

file <- script_file("warning")
cat("script_file:", file, "\n")
stopifnot(endsWith(file, "test_script_file.R"))

path <- script_path("warning")
cat("script_path:", path, "\n")
# stopifnot(endsWith(path, "tests"))

# save.image("test_script_file.Rda")

## Regression test for bug reported by John Nolan:
##
## Whenever the pair (x[i], a[i]) == (0,1), NA would be returned, due
## to an internal computation of ( 0 * -Inf ) => NaN
##
## The code now checks for this particular issue and sets the value of
## ( 0 * -Inf ) to 0, which is correct for this calculation.
##
library(gtools)

x <- c(0, 0, 1)
alpha <- c(1, 2, 3)

stopifnot(ddirichlet(x = x, alpha = alpha) == 0)

library(gtools)

df1 <- data.frame(a = 1, b = 2, d = TRUE)
df2 <- data.frame(b = 7, c = "YES", d = FALSE)
df3 <- data.frame(b = 7, c = "YES")

smartbind(df1, df2, df3)
smartbind(df1 = df1, df2, df3 = df3)

library("gtools")

host <- "www.r-project.org"
host_index <- sprintf("http://%s/index.html", host)

## check that network/CRAN are available
rproj_OK <- try(scan(host_index, character(1), quiet = TRUE))

if (!inherits(rproj_OK, "try-error")) {
    socket <- make.socket(host, 80)
    print(socket)

    setTCPNoDelay(socket, TRUE)
    
    write.socket(socket, sprintf("GET %s\r\n", host_index))
    write.socket(socket, "HOST 127.0.0.1\r\n")
    write.socket(socket, "\r\n")

    ## Read and display the response
    while ((str <- read.socket(socket)) > "") {
        cat(str)
    }

    close.socket(socket)
}

library(gtools)

today <- Sys.Date()
tenweeks <- seq(today, length.out = 10, by = "1 week")

df1 <- data.frame(dates = tenweeks, chars = letters[1:10], ints = 1:10, numeric = 1.1:10.1)
df2 <- data.frame(chars = letters[11:20], ints = 11:20, numeric = 11.1:20.1)

smartbind(df1, df2)

library(gtools)

##############################
### Examples from man page ###
##############################

### Toy examples

# search for x=10
s <- binsearch(function(x) x - 10, range = c(0, 20))
stopifnot(s$where == 10)

# search for x=10.1
s <- binsearch(function(x) x - 10.1, range = c(0, 20))
stopifnot(s$where == c(10, 11))

### Classical toy example

# binary search for the index of 'M' among the sorted letters
fun <- function(X) {
  ifelse(LETTERS[X] > "M", 1,
    ifelse(LETTERS[X] < "M", -1, 0)
  )
}

s <- binsearch(fun, range = 1:26)
stopifnot(LETTERS[s$where] == "M")

##################################
### Test boundary contiditions ###
##################################

s <- binsearch(fun = function(x) x - 10, range = c(1, 10))
with(s, stopifnot(where == 10, value == 0, flag == "Found"))

s <- binsearch(fun = function(x) x - 1, range = c(1, 10))
with(s, stopifnot(where == 1, value == 0, flag == "Found"))


checkWarning <- function(expr) {
  myEnv <- environment()

  catchWarning <- function(w) {
    assign("warningValue", w, pos = myEnv)
    invokeRestart("muffleWarning")
  }

  retval <- withCallingHandlers(
    expr = expr,
    warning = catchWarning
  )


  if (!exists("warningValue", where = myEnv, inherits = FALSE)) {
    stop("Expected a warning message")
  }
}

checkWarning(s <- binsearch(fun = function(x) x - 10, range = c(1, 9)))
with(s, stopifnot(where == 9, value == -1, flag == "Upper Boundary"))

checkWarning(s <- binsearch(fun = function(x) x - 1, range = c(2, 10)))
with(s, stopifnot(where == 2, value == 1, flag == "Lower Boundary"))

library(gtools)

Sys.setlocale(category = "LC_ALL", locale = "C")

## Examples from man page
Treatment <- c(
  "Control", "Asprin 10mg/day", "Asprin 50mg/day",
  "Asprin 100mg/day", "Acetomycin 100mg/day",
  "Acetomycin 1000mg/day"
)

stopifnot(mixedorder(Treatment) == c(5, 6, 2, 3, 4, 1))


x <- rev(c(
  "AA 0.50 ml", "AA 1.5 ml", "AA 500 ml", "AA 1500 ml",
  "EXP 1", "AA 1e3 ml", "A A A", "1 2 3 A", "NA", NA, "1e2",
  "", "-", "1A", "1 A", "100", "100A", "Inf"
))

stopifnot(mixedorder(x) == c(7, 11, 4, 5, 3, 8, 2, 1, 6, 12, 18, 17, 16, 13, 15, 14, 10, 9))

## Bug reported by Aaron Taudt on 2014-03-01

tmp <- c(
  "uniresult_simulated_H3k27ac_binsize_200_chr1.RData",
  "uniresult_simulated_H3k27me3_binsize_200_chr1.RData",
  "uniresult_simulated_H3k36me3_binsize_200_chr1.RData",
  "uniresult_simulated_H3k4me3_binsize_200_chr1.RData",
  "uniresult_simulated_H3k9me3_binsize_200_chr1.RData"
)

stopifnot(mixedorder(tmp) == c(4, 5, 1, 2, 3))

library(gtools)

Sys.setlocale(category = "LC_ALL", locale = "C")

## Examples from man page
Treatment <- c("Control", "Asprin 10mg/day", "Asprin 50mg/day",
               "Asprin 100mg/day", "Acetomycin 100mg/day",
               "Acetomycin 1000mg/day")

stopifnot( mixedorder(Treatment)==c(5, 6, 2, 3, 4, 1) )


x <- rev(c("AA 0.50 ml", "AA 1.5 ml", "AA 500 ml", "AA 1500 ml",
           "EXP 1", "AA 1e3 ml", "A A A", "1 2 3 A", "NA", NA, "1e2",
           "", "-", "1A", "1 A", "100", "100A", "Inf"))

stopifnot( mixedorder(x)==c(7, 11, 4, 5, 3, 8, 2, 1, 6, 12, 18, 17, 16, 13, 15, 14, 10, 9) )

## Bug reported by Aaron Taudt on 2014-03-01

tmp <- c("uniresult_simulated_H3k27ac_binsize_200_chr1.RData",
         "uniresult_simulated_H3k27me3_binsize_200_chr1.RData",
         "uniresult_simulated_H3k36me3_binsize_200_chr1.RData",
         "uniresult_simulated_H3k4me3_binsize_200_chr1.RData",
         "uniresult_simulated_H3k9me3_binsize_200_chr1.RData")

stopifnot( mixedorder(tmp)==c(4, 5, 1, 2, 3) )

#' Convert a byte-code function to an interpreted-code function
#'
#' The purpose of these functions is to allow a byte coded function to be
#' converted back into a fully interpreted function as a \emph{temporary} work
#' around for issues in byte-code interpretation.
#'
#' \code{unByteCode} returns a copy of the function that is directly
#' interpreted from text rather than from byte-code.
#'
#' \code{assignEdgewise} makes an assignment into a locked environment.
#'
#' \code{unByteCodeAssign} changes the specified function \emph{in its source
#' environment} to be directly interpreted from text rather than from
#' byte-code.
#'
#' The latter two functions no longer work out of the box because \code{assignEdgewise}
#' (which \code{unByteCodeAssign} uses) makes use of an unsafe \code{unlockBinding}
#' call, but running \code{assignEdgewise()} will 
#'
#' @aliases unByteCode unByteCodeAssign assignEdgewise
#' @param fun function to be modified
#' @param name object name
#' @param env namespace
#' @param value new function body
#' @return All three functions return a copy of the modified function or
#' assigned value.
#' @note These functions are not intended as a permanent solution to issues
#' with byte-code compilation or interpretation.  Any such issues should be
#' promptly reported to the R maintainers via the R Bug Tracking System at
#' \url{https://bugs.r-project.org} and via the R-devel mailing list
#' \url{https://stat.ethz.ch/mailman/listinfo/r-devel}.
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link[compiler]{disassemble}}, \code{\link{assign}}
#' @references These functions were inspired as a work-around to R bug
#' \url{https://bugs.r-project.org/show_bug.cgi?id=15215}.
#' @keywords programming utilites
#' @examples
#'
#' data(badDend)
#' dist2 <- function(x) as.dist(1 - cor(t(x), method = "pearson"))
#' hclust1 <- function(x) hclust(x, method = "single")
#'
#' distance <- dist2(badDend)
#' cluster <- hclust1(distance)
#'
#' dend <- as.dendrogram(cluster)
#' \dontrun{
#' ## In R 2.3.0 and earlier crashes with a node stack overflow error
#' plot(dend)
#' ## Error in xy.coords(x, y, recycle = TRUE) : node stack overflow
#' }
#'
#' ## convert stats:::plotNode from byte-code to interpreted-code
#' ## (no longer available unless assignEdgewise is defined by the user)
#' ## unByteCodeAssign(stats:::plotNode)
#' ## illustrated in https://stackoverflow.com/questions/16559250/error-in-heatmap-2-gplots
#'
#' # increase recursion limit
#' options("expressions" = 5e4)
#'
#' # now the function does not crash
#' plot(dend)
#' @export
unByteCode <- function(fun) {
  FUN <- eval(parse(text = deparse(fun)))
  environment(FUN) <- environment(fun)
  FUN
}

#' @rdname unByteCode
#' @export
assignEdgewise <- function(name, env, value) {
    stop("assignEdgewise no longer works due to unsafe use of ",
         sQuote("unlockBinding"),
         ". You can define this function yourself: ",
         "\n\nassignEdgewise <- function(name, env, value) {",
         "\n  unlockBinding(name, env = env)",
         "\n  assign(name, envir = env, value = value)",
         "\n  lockBinding(name, env = env)",
         "\n  invisible(value)",
         "\n}")
}

#' @rdname unByteCode
#' @export
unByteCodeAssign <- function(fun) {
  name <- gsub("^.*::+", "", deparse(substitute(fun)))
  FUN <- unByteCode(fun)
  retval <- assignEdgewise(
    name = name,
    env = environment(FUN),
    value = FUN
  )
  invisible(retval)
}

#' Non-destructively construct a .Last function to be executed when R exits.
#'
#' Non-destructively construct a \code{.Last} function to be executed when R
#' exits.
#'
#' \code{lastAdd} constructs a new function which can be used to replace the
#' existing definition of \code{.Last}, which will be executed when R terminates
#' normally.
#'
#' If a \code{.Last} function already exists in the global environment, the
#' original definition is stored in a private environment, and the new function
#' is defined to call the function \code{fun} and then to call the previous
#' (stored) definition of \code{.Last}.
#'
#' If no \code{.Last} function exists in the global environment, \code{lastAdd}
#' simply returns the function \code{fun}.
#'
#' @param fun Function to be called.
#' @return A new function to be used for \code{.Last}.
#' @note This function replaces the (now defunct) \code{addLast} function.
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link[base]{.Last}}
#' @keywords programming
#' @examples
#'
#'
#' ## Print a couple of cute messages when R exits.
#' helloWorld <- function() cat("\nHello World!\n")
#' byeWorld <- function() cat("\nGoodbye World!\n")
#'
#' .Last <- lastAdd(byeWorld)
#' .Last <- lastAdd(helloWorld)
#' \dontshow{
#' .Last()
#' }
#' \dontrun{
#' q("no")
#'
#' ## Should yield:
#' ##
#' ##   Save workspace image? [y/n/c]: n
#' ##
#' ##   Hello World!
#' ##
#' ##   Goodbye World!
#' ##
#' ##   Process R finished at Tue Nov 22 10:28:55 2005
#' }
#'
#' @export
lastAdd <- function(fun) {
  if (!is.function(fun)) stop("fun must be a function")

  if (!exists(".Last", envir = .GlobalEnv, mode = "function")) {
    return(fun)
  }
  else {
    Last <- get(".Last", envir = .GlobalEnv, mode = "function")
    newfun <- function(...) {
      fun()
      Last()
    }
    return(newfun)
  }
}

#' Compute fold-change or convert between log-ratio and fold-change.
#'
#' \code{foldchange} computes the fold change for two sets of values.
#' \code{logratio2foldchange} converts values from log-ratios to fold changes.
#' \code{foldchange2logratio} does the reverse.
#'
#' Fold changes are commonly used in the biological sciences as a mechanism for
#' comparing the relative size of two measurements.  They are computed as:
#' \eqn{\frac{num}{denom}}{num/denom} if \eqn{num>denom}, and as
#' \eqn{\frac{-denom}{num}}{-denom/num} otherwise.
#'
#' Fold-changes have the advantage of ease of interpretation and symmetry about
#' \eqn{num=denom}, but suffer from a discontinuity between -1 and 1, which can
#' cause significant problems when performing data analysis.  Consequently
#' statisticians prefer to use log-ratios.
#'
#' @aliases foldchange logratio2foldchange foldchange2logratio
#' @param num,denom vector/matrix of numeric values
#' @param logratio vector/matrix of log-ratio values
#' @param foldchange vector/matrix of fold-change values
#' @param base Exponential base for the log-ratio.
#' @return A vector or matrix of the same dimensions as the input containing
#' the converted values.
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @keywords math
#' @examples
#'
#'
#' a <- 1:21
#' b <- 21:1
#'
#' f <- foldchange(a, b)
#'
#' cbind(a, b, f)
#' @name foldchange
NULL

#' @describeIn foldchange Compute fold-change.
#' @export
foldchange <- function(num, denom) {
  ifelse(num >= denom, num / denom, -denom / num)
}

#' @describeIn foldchange Compute foldchange from log-ratio values.
#' @export
logratio2foldchange <- function(logratio, base = 2) {
  retval <- base^(logratio)
  retval <- ifelse(retval < 1, -1 / retval, retval)
  retval
}

#' @describeIn foldchange Compute log-ratio from fold-change values.
#' @export
foldchange2logratio <- function(foldchange, base = 2) {
  retval <- ifelse(foldchange < 0, 1 / -foldchange, foldchange)
  retval <- log(retval, base)
  retval
}

#' List valid keywords for R man pages
#'
#' List valid keywords for R man pages
#'
#' If \code{topic} is provided, return a list of the keywords associated with
#' \code{topic}.  Otherwise, display the list of valid R keywords from the R
#' doc/KEYWORDS file.
#'
#' @param topic object or man page topic
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link[utils]{help}}
#' @keywords documentation
#' @examples
#'
#' ## Show all valid R keywords
#' \dontrun{
#' keywords()
#'
#' ## Show keywords associated with the 'merge' function
#' keywords(merge)
#' keywords("merge")
#' }
#' @importFrom stats na.omit
#' @importFrom utils help.search
#' @export
keywords <- function(topic) {
  file <- file.path(R.home("doc"), "KEYWORDS")
  if (missing(topic)) {
    file.show(file)
  }
  else {
    kw <- scan(file = file, what = character(), sep = "\n", quiet = TRUE)
    kw <- grep("&", kw, value = TRUE)
    kw <- gsub("&[^&]*$", "", kw)
    kw <- gsub("&+", " ", kw)
    kw <- na.omit(trimws(kw))

    ischar <- tryCatch(is.character(topic) && length(topic) ==
      1L, error = identity)
    if (inherits(ischar, "error")) {
      ischar <- FALSE
    }
    if (!ischar) {
      topic <- deparse(substitute(topic))
    }

    item <- paste("^", topic, "$", sep = "")

    topics <- function(k) {
      matches <- help.search(keyword = k)$matches
      matches[, match("topic", tolower(colnames(matches)))]
    }
    matches <- lapply(kw, topics)
    names(matches) <- kw

    tmp <- unlist(lapply(matches, function(m) grep(item, m, value = TRUE)))
    names(tmp)
  }
}

testConvert <- function() {
  roman <- "IVXLCDM"
  retval <- romandigit.convert(roman)
  stopifnot(retval == c(1, 5, 10, 50, 100, 500, 1000))
  return(TRUE)
}

romandigit.convert <- function(roman) {
  retval <- .C(C_convert,
    roman = as.character(roman),
    nchar = as.integer(nchar(roman)),
    values = integer(nchar(roman)),
    PACKAGE = "gtools"
  )
  retval$values
}

roman2int.inner <- function(roman) {
  results <- .C(C_roman2int,
    roman = as.character(roman),
    nchar = as.integer(nchar(roman)),
    value = integer(1),
    PACKAGE = "gtools"
  )

  return(results$value)
}



#' Convert Roman Numerals to Integers
#'
#' Convert roman numerals to integers
#'
#' This function will convert roman numerals to integers without the upper bound
#' imposed by R (3899), ignoring case.
#'
#' @param roman character vector containing roman numerals
#' @return A integer vector with the same length as \code{roman}.  Character
#' strings which are not valid roman numerals will be converted to \code{NA}.
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link[utils]{as.roman}}
#' @keywords arith
#' @examples
#'
#' roman2int(c("I", "V", "X", "C", "L", "D", "M"))
#'
#' # works regardless of case
#' roman2int("MMXVI")
#' roman2int("mmxvi")
#'
#' # works beyond R's limit of 3899
#' val.3899 <- "MMMDCCCXCIX"
#' val.3900 <- "MMMCM"
#' val.4000 <- "MMMM"
#' as.numeric(as.roman(val.3899))
#' as.numeric(as.roman(val.3900))
#' as.numeric(as.roman(val.4000))
#'
#' roman2int(val.3899)
#' roman2int(val.3900)
#' roman2int(val.4000)
#' @export
roman2int <- function(roman) {
  roman <- trimws(toupper(as.character(roman)))

  tryIt <- function(x) {
    retval <- try(roman2int.inner(x), silent = TRUE)
    if (is.numeric(retval)) {
      retval
    } else {
      NA
    }
  }

  retval <- sapply(roman, tryIt)

  retval
}

## https://stackoverflow.com/questions/73491296/is-there-an-alternative-to-reshape2melt-for-multidimensional-arrays-in-base-r/76388312?noredirect=1#comment134733759_76388312

#' @param x a multidimensional array
#' @examples
#' a <- array(1:27, dim = c(3,3,3),
#'      dimnames = list(dim1 = letters[1:3], dim2 = LETTERS[1:3],
#'                      dim3 = c("i", "ii", "iii")))
#' melt_array(a)
melt_array <- function(x) {
    data.frame(arrayInd(seq_along(x), dim(x)), value = as.vector(x))
    ## list2DF(lapply(as.data.frame.table(x), unclass))
    ## list2DF(c(Map(
    ##     \(i, j, n) rep(rep(1:i, each = j), length.out = n),
    ##     dim(x),
    ##     c(1, cumprod(dim(x)[-length(dim(x))])),
    ##     length(x)
    ## ), Value = list(as.vector(x))))
}

#' Functions for the Dirichlet Distribution
#'
#' Functions to compute the density of or generate random deviates from the
#' Dirichlet distribution
#'
#' The Dirichlet distribution is the multidimensional generalization of the
#' beta distribution.  It is the canonical Bayesian distribution for the
#' parameter estimates of a multinomial distribution.
#'
#' @aliases rdirichlet ddirichlet
#' @param x A vector containing a single random deviate or matrix containing one
#' random deviate per row.
#' @param alpha Vector or (for \code{ddirichlet}) matrix containing shape
#' parameters.
#' @return \code{ddirichlet} returns a vector containing the Dirichlet density
#' for the corresponding rows of \code{x}.
#'
#' \code{rdirichlet} returns a matrix with \code{n} rows, each containing a
#' single Dirichlet random deviate.
#' @author Code original posted by Ben Bolker to R-News on Fri Dec 15 2000. See
#' \url{https://stat.ethz.ch/pipermail/r-help/2000-December/009561.html}.  Ben
#' attributed the code to Ian Wilson \email{i.wilson@@maths.abdn.ac.uk}.
#' Subsequent modifications by Gregory R. Warnes \email{greg@@warnes.net}.
#' @seealso \code{\link{dbeta}}, \code{\link{rbeta}}
#' @keywords distribution
#' @examples
#'
#'
#' x <- rdirichlet(20, c(1, 1, 1))
#'
#' ddirichlet(x, c(1, 1, 1))
#' @name dirichlet
NULL

#' @describeIn dirichlet  Dirichlet distribution function.
#' @export
ddirichlet <- function(x, alpha) {
  dirichlet1 <- function(x, alpha) {
    logD <- sum(lgamma(alpha)) - lgamma(sum(alpha))
    s <- (alpha - 1) * log(x)
    s <- ifelse(alpha == 1 & x == 0, -Inf, s)
    exp(sum(s) - logD)
  }

  # make sure x is a matrix
  if (!is.matrix(x)) {
    if (is.data.frame(x)) {
      x <- as.matrix(x)
    } else {
      x <- t(x)
    }
  }

  if (!is.matrix(alpha)) {
    alpha <- matrix(alpha, ncol = length(alpha), nrow = nrow(x), byrow = TRUE)
  }

  if (any(dim(x) != dim(alpha))) {
    stop("Mismatch between dimensions of 'x' and 'alpha'.")
  }

  pd <- vector(length = nrow(x))
  for (i in 1:nrow(x)) {
    pd[i] <- dirichlet1(x[i, ], alpha[i, ])
  }

  # Enforce 0 <= x[i,j] <= 1, sum(x[i,]) = 1
  pd[apply(x, 1, function(z) any(z < 0 | z > 1))] <- 0
  pd[apply(x, 1, function(z) all.equal(sum(z), 1) != TRUE)] <- 0
  pd
}

#' @describeIn dirichlet Generate dirichlet random deviates.
#' @param n Number of random vectors to generate.
#' @importFrom stats rgamma
#' @export
rdirichlet <- function(n, alpha) {
  l <- length(alpha)
  x <- matrix(rgamma(l * n, alpha), ncol = l, byrow = TRUE)
  sm <- x %*% rep(1, l)
  x / as.vector(sm)
}

#' Generalized logit and inverse logit function
#'
#' Compute generalized logit and generalized inverse logit functions.
#'
#'
#' The generalized logit function takes values on [min, max] and transforms
#' them to span [-Inf,Inf] it is defined as:
#'
#' \deqn{y = log(\frac{p}{(1-p)})}{y = log(p/(1-p))}
#'
#' where
#'
#' \deqn{p=\frac{(x-min)}{(max-min)}}{p=(x-min)/(max-min)}
#'
#' The generalized inverse logit function provides the inverse transformation:
#'
#' \deqn{x = p' (max-min) + min}{x = p * (max-min) + min}
#'
#' where
#'
#' \deqn{p'=\frac{exp(y)}{(1+exp(y))}}{exp(y)/(1+exp(y))}
#'
#' @aliases logit inv.logit
#' @param x value(s) to be transformed
#' @param min Lower end of logit interval
#' @param max Upper end of logit interval
#' @return Transformed value(s).
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link[car]{logit}}
#' @keywords math
#' @examples
#'
#'
#' x <- seq(0, 10, by = 0.25)
#' xt <- logit(x, min = 0, max = 10)
#' cbind(x, xt)
#'
#' y <- inv.logit(xt, min = 0, max = 10)
#' cbind(x, xt, y)
#' @export
logit <- function(x, min = 0, max = 1) {
  p <- (x - min) / (max - min)
  log(p / (1 - p))
}


#' @rdname logit
#' @export
inv.logit <- function(x, min = 0, max = 1) {
  p <- exp(x) / (1 + exp(x))
  p <- ifelse(is.na(p) & !is.na(x), 1, p) # fix problems with +Inf
  p * (max - min) + min
}

#' Transform an integer to an array of base-n digits
#'
#' Transform an integer to an array of base-n digits
#'
#' This function converts the elements of an integer vector as an array of its
#' digits.  The base of the numbering scheme may be changed away from 10, which
#' defines our decimal system, to any other integer value. For base=2, the
#' number is returned in the binary system. The least significant digit has the
#' highest index in the array, i.e. it appears on the right.  The highest
#' exponent is at position 1, i.e. left.
#'
#' To write decimal values in another base is very common in computer science.
#' In particular at the basis 2 the then possible values 0 and 1 are often
#' interpreted as logical false or true. And at the very interface to
#' electrical engineering, it is indicated as an absence or presence of
#' voltage. When several bit values are transported synchronously, then it is
#' common to give every lane of such a data bus a unique 2^x value and
#' interpret it as a number in the binary system. To distinguish 256 characters
#' one once needed 8 bit ("byte"). It is the common unit in which larger
#' non-printable data is presented.  Because of the many non-printable
#' characters and the difficulty for most humans to memorize an even longer
#' alphabet, it is presented as two half bytes ("nibble") of 4 bit in a
#' hexadecimal presentation. Example code is shown below.
#'
#' For statisticians, it is more likely to use bit representations for hashing.
#' A bit set to 1 (TRUE) at e.g. position 2, 9 or 17 is interpreted as the
#' presence of a particular feature combination of a sample.  With baseOf, you
#' can refer to the bit combination as a number, which is more easily and more
#' efficiently dealt with than with an array of binary values. The example code
#' presents a counter of combinations of features which may be interpreted as a
#' Venn diagram.
#'
#' @param v A single integer value to be transformed.
#' @param base The base to which to transform to.
#' @param len The minimal length of the returned array.
#' @author Steffen Moeller \email{moeller@@debian.org}
#' @keywords base
#' @examples
#'
#' # decimal representation
#' baseOf(123)
#'
#' # binary representation
#' baseOf(123, base = 2)
#'
#' # octal representation
#' baseOf(123, base = 8)
#'
#' # hexadecimal representation
#' baseOf(123, base = 16)
#'
#' # hexadecimal with more typical letter-notation
#' c(0:9, LETTERS)[baseOf(123, 16)]
#'
#' # hexadecimal again, now showing a single string
#' paste(c(0:9, LETTERS)[baseOf(123, 16)], collapse = "")
#'
#' # decimal representation but filling leading zeroes
#' baseOf(123, len = 5)
#'
#' # and converting that back
#' sum(2^(4:0) * baseOf(123, len = 5))
#'
#' # hashing and a tabular venn diagram derived from it
#' m <- matrix(sample(c(FALSE, TRUE), replace = TRUE, size = 300), ncol = 4)
#' colnames(m) <- c("strong", "colorful", "nice", "humorous")
#' names(dimnames(m)) <- c("samples", "features")
#' head(m)
#'
#' m.val <- apply(m, 1, function(X) {
#'   return(sum(2^((ncol(m) - 1):0) * X))
#' })
#' m.val.rle <- rle(sort(m.val))
#' m.counts <- cbind(
#'   baseOf(m.val.rle$value, base = 2, len = ncol(m)),
#'   m.val.rle$lengths
#' )
#' colnames(m.counts) <- c(colnames(m), "num")
#' rownames(m.counts) <- apply(m.counts[, 1:ncol(m)], 1, paste, collapse = "")
#' m.counts[1 == m.counts[, "nice"] & 1 == m.counts[, "humorous"], , drop = FALSE]
#' m.counts[, "num", drop = TRUE]
#' @export
baseOf <- function(v,
                   base = 10,
                   len = 1) {
  if (is.null(v)) {
    stop("v is null")
  }
  if (length(v) == 0) {
    return(integer(0))
  }

  if (any(as.integer(v) != v)) {
    stop("non-integer value(s) provided for v.")
  }

  if (length(v) > 1) {
    # this returns a list which may have vectors of varying lenths
    val.list <- lapply(X = v, FUN = baseOf.inner, base = base, len = len)
    longest <- max(sapply(val.list, length))

    # call again, forcing all elements to have the same lenth
    retval <- t(sapply(X = v, FUN = baseOf.inner, base = base, len = longest))

    # add informative row and column names
    rownames(retval) <- paste0("v.", v)
    colnames(retval) <- paste0("b.", rev(c(0, base^(1:(longest - 1)))))

    retval
  }
  else {
    retval <- baseOf.inner(v = v, base = base, len = len)
  }

  retval
}


# Transform integer to array of digits in specified
baseOf.inner <- function(v,
                         base = 10,
                         len = 1) {
  if (is.na(v)) {
    return(rep(NA, len))
  }

  if (v == 0) {
    return(rep(0, len))
  }

  remainder <- v
  i <- len
  ret <- NULL
  while (remainder > 0 || i > 0) {
    # print(paste("i=",i," remainder=",remainder))
    m <- remainder %% base
    if (is.null(ret)) {
      ret <- m
    }
    else {
      ret <- c(m, ret)
    }
    remainder <- remainder %/% base
    i <- i - 1
  }

  if (length(ret) > 1) {
    names(ret) <- rev(c(0, base^(1:(length(ret) - 1))))
  }

  return(ret)
}

## trimws was added in R 2.3.0.  If we're using a previous version of
## R we need to define it.
if (!exists("trimws", mode = "function")) {
  trimws <- function(s) {
    s <- sub(pattern = "^[[:blank:]]+", replacement = "", x = s)
    s <- sub(pattern = "[[:blank:]]+$", replacement = "", x = s)
    s
  }
}

#' Modify the TCP_NODELAY (`de-Nagle') flag for socket objects
#'
#' Modify the TCP_NODELAY (`de-Nagle') flag for socket objects
#'
#' By default, TCP connections wait a small fixed interval before actually
#' sending data, in order to permit small packets to be combined.  This
#' algorithm is named after its inventor, John Nagle, and is often referred to
#' as 'Nagling'.
#'
#' While this reduces network resource utilization in these situations, it
#' imposes a delay on all outgoing message data, which can cause problems in
#' client/server situations.
#'
#' This function allows this feature to be disabled (de-Nagling,
#' \code{value=TRUE}) or enabled (Nagling, \code{value=FALSE}) for the
#' specified socket.
#'
#' @param socket A socket connection object
#' @param value Logical indicating whether to set (\code{TRUE}) or unset
#' (\code{FALSE}) the flag
#'
#' @return The character string "SUCCESS" will be returned invisible if the
#' operation was successful.  On failure, an error will be generated.
#'
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#'
#' @seealso \code{\link{make.socket}}, \code{\link{socketConnection}}
#'
#' @references
#'   "Nagle's algorithm" \url{https://en.wikipedia.org/wiki/Nagle's_algorithm},
#'
#'   Nagle, John. "Congestion Control in IP/TCP Internetworks", IETF Request for
#'   Comments 896, January 1984. \url{https://www.ietf.org/rfc/rfc0896.txt?number=896}
#'
#' @keywords programming misc utilities
#' @examples
#' \dontrun{
#' host <- "www.r-project.org"
#' socket <- make.socket(host, 80)
#' print(socket)
#' setTCPNoDelay(socket, TRUE)
#'
#' write.socket(socket, "GET /\n\n")
#' write.socket(socket, "A")
#' write.socket(socket, "B\n")
#' while ((str <- read.socket(socket)) > "") {
#'   cat(str)
#' }
#' close.socket(socket)
#' }
#'
#' @export
setTCPNoDelay <- function(socket, value = TRUE) {
  if (!any(c("socket", "sockconn") %in% class(socket))) {
    stop("socket must be a socket object")
  }

  buffer <- paste(rep(" ", 1000), sep = "", collapse = "")

  if ("sockconn" %in% class(socket)) {
    conn <- getConnection(socket[1])
  } else {
    conn <- socket
  }

  retval <- .C(C_setTCPNoDelay,
    socket = as.integer(socket[1]),
    flag = as.integer(value),
    status = integer(1),
    status.str = as.character(buffer),
    status.len = as.integer(nchar(buffer)),
    PACKAGE = "gtools"
  )

  if (retval$status != 0) {
    stop(retval$status.str)
  } else {
    invisible(retval$status.str)
  }
}

library(gtools)

df1 <- data.frame(a=1, b=2,          d=TRUE )
df2 <- data.frame(     b=7, c="YES", d=FALSE)
df3 <- data.frame(     b=7, c="YES"         )

smartbind( df1, df2, df3 )
smartbind( df1=df1, df2, df3=df3 )

#' @rdname defmacro
#' @export
strmacro <- function(..., expr, strexpr) {
  if (!missing(expr)) {
    strexpr <- deparse(substitute(expr))
  }

  a <- substitute(list(...))[-1]

  nn <- names(a)
  if (is.null(nn)) {
    nn <- rep("", length(a))
  }
  for (i in 1:length(a))
  {
    if (nn[i] == "") {
      nn[i] <- paste(a[[i]])
      msg <- paste(a[[i]], "not supplied")
      a[[i]] <- substitute(
        stop(foo),
        list(foo = msg)
      )
    }
    else {
      a[[i]] <- a[[i]]
    }
  }
  names(a) <- nn
  a <- as.list(a)

  ## this is where the work is done
  ff <-
    function(...) {
      ## build replacement list
      reptab <- a # copy defaults first
      reptab$"..." <- NULL

      args <- match.call(expand.dots = TRUE)[-1]

      for (item in names(args)) {
        reptab[[item]] <- args[[item]]
      }

      ## do the replacements
      body <- strexpr
      for (i in 1:length(reptab))
      {
        pattern <- paste("\\b",
          names(reptab)[i],
          "\\b",
          sep = ""
        )

        value <- reptab[[i]]
        if (missing(value)) {
          value <- ""
        }

        body <- gsub(
          pattern,
          value,
          body
        )
      }

      fun <- parse(text = body)
      eval(fun, parent.frame())
    }



  ## add the argument list
  formals(ff) <- a

  ## create a fake source attribute
  mm <- match.call()
  mm$expr <- NULL
  mm[[1]] <- as.name("macro")
  attr(ff, "source") <- c(deparse(mm), strexpr)

  ## return the 'macro'
  ff
}

#' Capitalize Words for Titles
#'
#' This function capitalizes words for use in titles
#'
#' This function separates the provided character string into separate words
#' using \code{sep} as the word separator.  If \code{firstonly==TRUE}, it then
#' capitalizes the first letter the first word, otherwise (the default), it
#' capitalizes the first letter of every word.  If \code{AP==TRUE}, it then
#' un-capitalizes words in the Associated Press's (AP) list of prepositions and
#' conjunctions should not be capitalized in titles.  Next, it capitalizes the
#' first word. It then re-joins the words using the specified separator.
#'
#' If \code{preserveMixed==TRUE}, words with an upper-case letter appearing
#' after a lower-case letter will not be changed (e.g. "iDevice").
#'
#' @param s character string to be processed
#' @param strict Logical, remove all additional capitalization.
#' @param AP Logical, apply the Associated Press (AP) rules for prepositions
#' and conjunctions that should not be capitalized in titles.
#' @param onlyfirst Logical, only capitalize the first word.
#' @param preserveMixed Logical, preserve the capitalization mixed-case words
#' containing an upper-case letter after a lower-case letter.
#' @param sep Character string, word separator
#' @return A character scalar containing the capitalized words.
#' @author Gregory R. Warnes \email{greg@@warnes.net} based on code from the
#' \code{\link[base]{chartr}} manual page, and Scott Chamberlain's
#' \code{\link[taxize]{taxize_capwords}} in the taxize package.
#' @seealso \code{\link[base]{chartr}}, \code{\link[taxize]{taxize_capwords}},
#' \code{\link[SGP]{capwords}}
#' @references Fogarty, Mignon. Capitalizing Titles: "Which words should you
#' capitalize? Grammar Girl's Quick and Dirty Tips for Better Writing. 9 Jun.
#' 2011. Quick and Dirty Tips Website." Accessed 22 April 2016
#' \url{https://www.quickanddirtytips.com/articles/capitalizing-titles/}
#' @keywords utilites character
#' @examples
#'
#' capwords("a function to capitalize words in a title")
#' capwords("a function to capitalize words in a title", AP = FALSE)
#'
#' capwords("testing the iProduct for defects")
#' capwords("testing the iProduct for defects", strict = TRUE)
#' capwords("testing the iProduct for defects", onlyfirst = TRUE)
#' capwords("testing the iProduct for defects", preserveMixed = TRUE)
#'
#' capwords("title_using_underscores_as_separators", sep = "_")
#' @export
capwords <- function(s,
                     strict = FALSE,
                     AP = TRUE,
                     onlyfirst = FALSE,
                     preserveMixed = FALSE,
                     sep = " ") {
  # worker functions
  cap <- function(s) {
    paste(toupper(substring(s, 1, 1)),
      {
        s <- substring(s, 2)
        if (strict) tolower(s) else s
      },
      sep = ""
    )
  }

  # test if there is a lowercase letter followed by an uppercase letter
  isMixedCase <- function(s) {
    grepl("[a-z][A-Z]", s)
  }

  words <- unlist(strsplit(s, split = sep))
  mixedCaseFlag <- sapply(words, isMixedCase)

  # First, capitalize *every* word
  if (!onlyfirst) {
    newWords <- sapply(
      words,
      cap
    )

    if (preserveMixed == TRUE) {
      newWords[mixedCaseFlag] <- words[mixedCaseFlag]
    }

    words <- newWords
  }

  # Next (optionally) uncapitalize prepositions and conjunctions
  # recommended by the Associated Press.
  AP.nocap <- c(
    "a", "an", "and", "at", "but", "by", "for", "in",
    "nor", "of", "on", "or", "so", "the", "to", "up",
    "yet"
  )

  if (AP && !onlyfirst) {
    for (word in AP.nocap) {
      words <- gsub(paste0("^", word, "$"),
        word,
        words,
        ignore.case = TRUE
      )
    }
  }


  # Finally, ensure that the first word is capitalized
  if (length(words) > 0 && mixedCaseFlag[1] == FALSE) {
    words[1] <- cap(words[1])
  }

  retval <- paste(words, collapse = sep)

  retval
}

#' Efficient rbind of data frames, even if the column names don't match
#'
#' Efficient rbind of data frames, even if the column names don't match
#'
#'
#' @param \dots Data frames to combine
#' @param list List containing data frames to combine
#' @param fill Value to use when 'filling' missing columns.  Defaults to
#' \code{NA}.
#' @param sep Character string used to separate column names when pasting them
#' together.
#' @param verbose Logical flag indicating whether to display processing
#' messages. Defaults to \code{FALSE}.
#' @return The returned data frame will contain: \item{columns}{all columns
#' present in any provided data frame} \item{rows}{a set of rows from each
#' provided data frame, with values in columns not present in the given data
#' frame filled with missing (\code{NA}) values.} The data type of columns will
#' be preserved, as long as all data frames with a given column name agree on
#' the data type of that column.  If the data frames disagree, the column will
#' be converted into a character strings.  The user will need to coerce such
#' character columns into an appropriate type.
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link{rbind}}, \code{\link{cbind}}
#' @keywords manip
#' @examples
#'
#'
#' df1 <- data.frame(A = 1:10, B = LETTERS[1:10], C = rnorm(10))
#' df2 <- data.frame(A = 11:20, D = rnorm(10), E = letters[1:10])
#'
#' # rbind would fail
#' \dontrun{
#' rbind(df1, df2)
#' # Error in match.names(clabs, names(xi)) : names do not match previous
#' # names:
#' # 	D, E
#' }
#' # but smartbind combines them, appropriately creating NA entries
#' smartbind(df1, df2)
#'
#' # specify fill=0 to put 0 into the missing row entries
#' smartbind(df1, df2, fill = 0)
#' \dontshow{
#' n <- 10 # number of data frames to create
#' s <- 10 # number of rows in each data frame
#'
#' # create a bunch of column names
#' names <- LETTERS[2:5]
#'
#' # create a list 'Z' containing 'n' data frames, each with 3 columns
#' # and 's' rows.  The first column is always named 'A', but the other
#' # two have a names randomly selected from 'names'
#'
#' Z <- list()
#' for (i in 1:n)
#' {
#'   X <- data.frame(
#'     A = sample(letters, s, replace = TRUE),
#'     B = letters[1:s],
#'     C = rnorm(s)
#'   )
#'   colnames(X) <- c("A", sample(names, 2, replace = FALSE))
#'   Z[[i]] <- X
#' }
#'
#' # Error in match.names(clabs, names(xi)) : names do not match
#' # previous names: E
#'
#' # But smartbind will 'do the right thing'
#' df <- do.call("smartbind", Z)
#' df
#'
#' # Equivalent call:
#' df <- smartbind(list = Z)
#' }
#'
#' @importFrom utils modifyList
#' @export
smartbind <- function(..., list, fill = NA, sep = ":", verbose = FALSE) {
  data <- base::list(...)
  if (!missing(list)) {
    data <- modifyList(list, data)
  }
  data <- data[!sapply(data, function(l) is.null(l) | (ncol(l) == 0) | (nrow(l) == 0))]


  defaultNames <- seq.int(length(data))

  if (is.null(names(data))) {
    names(data) <- defaultNames
  }

  emptyNames <- names(data) == ""
  if (any(emptyNames)) {
    names(data)[emptyNames] <- defaultNames[emptyNames]
  }

  data <- lapply(
    data,
    function(x) {
      if (is.matrix(x) || is.data.frame(x)) {
        x
      } else {
        data.frame(as.list(x), check.names = FALSE)
      }
    }
  )

  # retval <- new.env()
  retval <- base::list()
  rowLens <- unlist(lapply(data, nrow))
  nrows <- sum(rowLens)

  rowNameList <- unlist(lapply(
    names(data),
    function(x) {
      if (rowLens[x] <= 1) {
        x
      } else {
        paste(x, seq(1, rowLens[x]), sep = sep)
      }
    }
  ))

  colClassList <- vector(mode = "list", length = length(data))
  factorColumnList <- vector(mode = "list", length = length(data))
  factorLevelList <- vector(mode = "list", length = length(data))


  start <- 1
  blockIndex <- 1
  for (block in data)
  {
    colClassList[[blockIndex]] <- base::list()
    factorColumnList[[blockIndex]] <- character(length = 0)
    factorLevelList[[blockIndex]] <- base::list()

    if (verbose) print(head(block))
    end <- start + nrow(block) - 1
    for (col in colnames(block))
    {
      classVec <- class(block[, col])

      ## store class and factor level information for later use
      colClassList[[blockIndex]][[col]] <- classVec
      if ("factor" %in% classVec) {
        factorColumnList[[blockIndex]] <-
          c(factorColumnList[[blockIndex]], col)

        factorLevelList[[blockIndex]][[col]] <-
          levels(block[, col])
      }

      if (verbose) {
        cat("Start:", start,
          "  End:", end,
          "  Column:", col,
          "\n",
          sep = ""
        )
      }

      if ("factor" %in% classVec) {
        newclass <- "character"
      }
      else {
        newclass <- classVec[1]
      }

      ## Coerce everything that isn't a native type to character
      if (!(newclass %in% c(
        "logical", "integer", "numeric",
        "complex", "character", "raw"
      ))) {
        newclass <- "character"
        warning(
          "Converting non-atomic type column '", col,
          "' to type character."
        )
      }

      if (!(col %in% names(retval))) {
        retval[[col]] <- as.vector(rep(fill, nrows), mode = newclass)
      }

      ## Handle case when current and previous native types differ
      oldclass <- class(retval[[col]])

      if (oldclass != newclass) {
        # handle conversions in case of conflicts
        #   numeric vs integer --> numeric
        #   complex vs numeric or integer --> complex
        #   anything else:  --> character
        if (oldclass %in% c("integer", "numeric") && newclass %in% c("integer", "numeric")) {
          class(retval[[col]]) <- mode <- "numeric"
        } else if (oldclass == "complex" && newclass %in% c("integer", "numeric")) {
          class(retval[[col]]) <- mode <- "complex"
        } else if (oldclass %in% c("integer", "numeric") && newclass == "complex") {
          class(retval[[col]]) <- mode <- "complex"
        } else {
          class(retval[[col]]) <- mode <- "character"
          warning(
            "Column class mismatch for '", col, "'. ",
            "Converting column to class 'character'."
          )
        }
      }
      else {
        mode <- oldclass
      }

      if (mode == "character") {
        vals <- as.character(block[, col])
      } else {
        vals <- block[, col]
      }

      retval[[col]][start:end] <- as.vector(vals, mode = mode)
    }
    start <- end + 1
    blockIndex <- blockIndex + 1
  }

  all.equal.or.null <- function(x, y) {
    if (is.null(x) || is.null(y)) {
      return(TRUE)
    } else {
      return(all.equal(x, y))
    }
  }

  ## Handle factors, merging levels
  for (col in unique(unlist(factorColumnList)))
  {
    ## Ensure column classes match across blocks
    colClasses <- lapply(colClassList, function(x) x[[col]])
    firstNotNull <- which(!sapply(colClasses, is.null))[1]
    allSameOrNull <- all(sapply(
      colClasses[-firstNotNull],
      function(x) isTRUE(all.equal.or.null(colClasses[[firstNotNull]], x))
    ))

    if (allSameOrNull) {
      # grab the first *non-NULL* class information
      colClass <- colClasses[[firstNotNull]]
    }
    else {
      warning(
        "Column class mismatch for '", col, "'. ",
        "Converting column to class 'character'."
      )
      next()
    }


    ## check if factor levels are all the same
    colLevels <- lapply(factorLevelList, function(x) x[[col]])
    firstNotNull <- which(!sapply(colLevels, is.null))[1]
    allSameOrNull <- all(sapply(
      colLevels[-firstNotNull],
      function(x) isTRUE(all.equal.or.null(colLevels[[firstNotNull]], x))
    ))


    if (allSameOrNull) {
      if ("ordered" %in% colClass) {
        retval[[col]] <- ordered(retval[[col]], levels = colLevels[[firstNotNull]])
      } else {
        retval[[col]] <- factor(retval[[col]], levels = colLevels[[firstNotNull]])
      }
    }
    else {
      ## Check if longest set of levels is a superset of all others,
      ## and use that one
      longestIndex <- which.max(sapply(colLevels, length))
      longestLevels <- colLevels[[longestIndex]]
      allSubset <- all(sapply(
        colLevels[-longestIndex],
        function(l) all(l %in% longestLevels)
      ))
      if (allSubset) {
        if ("ordered" %in% colClass) {
          retval[[col]] <- ordered(retval[[col]], levels = longestLevels)
        } else {
          retval[[col]] <- factor(retval[[col]], levels = longestLevels)
        }
      }
      else {
        # form superset by appending to longest level set
        levelSuperSet <- unique(c(longestLevels, unlist(colLevels)))
        retval[[col]] <- factor(retval[[col]], levels = levelSuperSet)

        if (length(colClass) > 1) # not just plain factor
          {
            warning(
              "column '", col, "' of class ",
              paste("'", colClass, "'",
                collapse = ":",
                sep = "'"
              ),
              " converted to class 'factor'. Check level ordering."
            )
          }
      }
    }
  }

  attr(retval, "row.names") <- rowNameList
  class(retval) <- "data.frame"
  return(retval)
}

#' Convert Characters to ASCII
#'
#' Convert character vector to ASCII, replacing non-ASCII characters with
#' single-byte (\samp{\x00}) or two-byte (\samp{\u0000}) codes.
#'
#' @param x a character vector, possibly containing non-ASCII characters.
#' @param bytes either \code{1} or \code{2}, for single-byte (\samp{\x00}) or
#'        two-byte (\samp{\u0000}) codes.
#' @param fallback an output character to use, when input characters cannot be
#'        converted.
#'
#' @return
#' A character vector like \code{x}, except non-ASCII characters have been
#' replaced with \samp{\x00} or \samp{\u0000} codes.
#'
#' @note
#' To render single backslashes, use these or similar techniques: \preformatted{
#' write(ASCIIfy(x), "file.txt")
#' cat(paste(ASCIIfy(x), collapse="\n"), "\n", sep="")}
#'
#' The resulting strings are plain ASCII and can be used in R functions and
#' datasets to improve package portability.
#'
#' @author Arni Magnusson.
#'
#' @seealso
#' \code{\link[tools]{showNonASCII}} identifies non-ASCII characters in a
#' character vector.
#'
#' @keywords utilites character
#'
#' @examples
#' cities <- c("S\u00e3o Paulo", "Reykjav\u00edk")
#' print(cities)
#' ASCIIfy(cities, 1)
#' ASCIIfy(cities, 2)
#'
#' athens <- "\u0391\u03b8\u03ae\u03bd\u03b1"
#' print(athens)
#' ASCIIfy(athens)
#'
#' @export

ASCIIfy <- function(x, bytes = 2, fallback = "?") {
  bytes <- match.arg(as.character(bytes), 1:2)
  convert <- function(char) # convert to ASCII, e.g. "z", "\xfe", or "\u00fe"
  {
    raw <- charToRaw(char)
    if (length(raw) == 1 && raw <= 127) { # 7-bit
      ascii <- char
    } else if (length(raw) == 1 && bytes == 1) { # 8-bit to \x00
      ascii <- paste0("\\x", raw)
    } else if (length(raw) == 1 && bytes == 2) { # 8-bit to \u0000
      ascii <- paste0("\\u", chartr(" ", "0", formatC(as.character(raw), width = 4)))
    } else if (length(raw) == 2 && bytes == 1) { # 16-bit to \x00, if possible
      if (utf8ToInt(char) <= 255) {
        ascii <- paste0("\\x", format.hexmode(utf8ToInt(char)))
      } else {
        ascii <- fallback
        warning(char, " could not be converted to 1 byte")
      }
    } else if (length(raw) == 2 && bytes == 2) { # UTF-8 to \u0000
      ascii <- paste0("\\u", format.hexmode(utf8ToInt(char), width = 4))
    } else {
      ascii <- fallback
      warning(char, " could not be converted to ", bytes, " byte")
    }
    return(ascii)
  }

  if (length(x) > 1) {
    sapply(x, ASCIIfy, bytes = bytes, fallback = fallback, USE.NAMES = FALSE)
  }
  else {
    input <- unlist(strsplit(x, "")) # "c"  "a"  "f"  "<\'e>"
    output <- character(length(input)) # ""   ""   ""   ""
    for (i in seq_along(input)) {
      output[i] <- convert(input[i])
    } # "c"  "a"  "f"  "\\u00e9"
    output <- paste(output, collapse = "") # "caf\\u00e9"
    return(output)
  }
}

#' Data from an ELISA assay
#'
#' Observed signals and (for some observations) nominal concentrations for
#' samples that were aliquoted to multiple assay plates, which were read
#' multiple times on multiple days.
#'
#'
#' @name ELISA
#' @docType data
#' @format a data frame with the following columns:
#' \itemize{
#'   \item PlateDay factor. Specifies one of four physically distinct 96 well plates
#'   \item Read factor. The signal was read 3 times for each plate.
#'   \item Description character. Indicates contents of sample.
#'   \item Concentration numeric. Nominal concentration of standards (NA for all other samples).
#'   \item Signal numeric. Assay signal. Specifically, optical density (a colorimetric assay).
#' }
#' @source Anonymized data.
#' @keywords datasets
NULL

library(gtools)

##############################
### Examples from man page ###
##############################

### Toy examples

# search for x=10
s <- binsearch( function(x) x-10, range=c(0,20) )
stopifnot(s$where==10)

# search for x=10.1
s <- binsearch( function(x) x-10.1, range=c(0,20) )
stopifnot( s$where==c(10,11) )

### Classical toy example

# binary search for the index of 'M' among the sorted letters
fun <- function(X) ifelse(LETTERS[X] > 'M', 1,
                          ifelse(LETTERS[X] < 'M', -1, 0 ) )

s = binsearch( fun, range=1:26 )
stopifnot( LETTERS[s$where]=="M")

##################################
### Test boundary contiditions ###
##################################

s = binsearch(fun = function(x) x-10, range=c(1,10) )
with(s, stopifnot(where==10, value==0, flag=="Found") )

s = binsearch(fun = function(x) x-1, range=c(1,10) )
with(s, stopifnot(where==1, value==0, flag=="Found") )


checkWarning <- function( expr )
    {
        myEnv <- environment()

        catchWarning <- function(w) {
            assign("warningValue", w, pos=myEnv)
            invokeRestart("muffleWarning")
        }

        retval <- withCallingHandlers(expr = expr,
                                      warning = catchWarning)


        if( !exists("warningValue", where=myEnv, inherits=FALSE) )
            stop("Expected a warning message")
    }

checkWarning( s <- binsearch(fun = function(x) x-10, range=c(1,9) ) )
with(s, stopifnot(where==9, value==-1, flag=="Upper Boundary" ) )

checkWarning( s <- binsearch(fun = function(x) x-1, range=c(2,10) ) )
with(s, stopifnot(where==2, value==1, flag=="Lower Boundary" ) )

#' Test if a value is missing, empty, contains only NA or NULL values, or is a try-error.
#'
#' Test if a value is missing, empty, contains only NA or NULL values, or is a try-error.
#'
#'
#' @param x value to be tested
#' @return Logical value.
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link[base]{missing}}, \code{\link[base]{is.na}},
#' \code{\link[base]{is.null}}
#' @keywords programming
#' @examples
#'
#'
#' invalid(NA)
#' invalid()
#' invalid(c(NA, NA, NULL, NA))
#'
#' invalid(list(a = 1, b = NULL))
#'
#' x <- try(log("A"))
#' invalid(x)
#'
#' # example use in a function
#' myplot <- function(x, y) {
#'   if (invalid(y)) {
#'     y <- x
#'     x <- 1:length(y)
#'   }
#'   plot(x, y)
#' }
#' myplot(1:10)
#' myplot(1:10, NA)
#' @importFrom methods is
#'
#' @export
invalid <- function(x) {
  if (missing(x) || is.null(x) || length(x) == 0 || is(x, "try-error")) {
    return(TRUE)
  }
  if (is.list(x)) {
    return(all(sapply(x, invalid)))
  } else if (is.vector(x)) {
    return(all(is.na(x)))
  } else {
    return(FALSE)
  }
}

#' Randomly Permute the Elements of a Vector
#'
#' Randomly Permute the elements of a vector
#'
#' This is simply a wrapper function for \code{\link{sample}}.
#'
#' @param x Vector of items to be permuted
#' @return Vector with the original items reordered.
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link{sample}}
#' @keywords distribution
#' @examples
#'
#' x <- 1:10
#' permute(x)
#' @export
permute <- function(x) sample(x, size = length(x), replace = FALSE)

#' Determine the directory or full path to the currently executing script
#'
#' @param fail character, one of "stop", "warning", "quiet". specifying what
#'  should be done when the script path cannot be determined:
#'  "stop" causes an error to be generated,
#'  "warn" generates a warning message and returns NA,
#'  "quiet" silently returns NA.
#'
#' These function should work with \code{Rscript}, \code{source()},
#' \code{Rmarkdown} \code{RStudio}'s "Run selection", and the \code{RStudio}
#' Console.
#'
#' @author Greg Warnes \email{greg@warnes.net} based on on a Stack Overflow post by
#'   jerry-t (\url{https://stackoverflow.com/users/2292993/jerry-t}) at
#'   \url{https://stackoverflow.com/a/36777602/2744062}.
#'
#' @return A character scalar containing the full path to the currently
#'   executing script file (\code{script_file}) or its directory
#'   (\code{script_path}).  If unable to determine the script path, it generates
#'   a warning and returns \code{""} (empty string).
#'
#' @examples
#'
#' getwd()
#' commandArgs(trailingOnly = FALSE)
#'
#' script_file("warning")
#' script_path("warning")
#' @name script_file
NULL

#' @describeIn script_file  Determine the full path of the currently executing
#'   script
#' @export
script_file <- function(fail = c("stop", "warning", "quiet")) {
  fail <- match.arg(fail)

  # https://stackoverflow.com/a/32016824/2292993
  cmdArgs <- commandArgs(trailingOnly = FALSE)
  needle <- "--file="
  match <- grep(needle, cmdArgs)
  if (length(match) > 0) {
    # Rscript via command line with "--file="
    return(normalizePath(sub(needle, "", cmdArgs[match])))
  } else if ("-f" %in% cmdArgs) {
    idx <- which("-f" == cmdArgs) + 1
    return(normalizePath(cmdArgs[idx]))
  } else {
    ls_vars <- ls(sys.frames()[[1]])
    if ("fileName" %in% ls_vars) {
      # Source'd via RStudio
      return(normalizePath(sys.frames()[[1]]$fileName))
    } else {
      if (!is.null(sys.frames()[[1]]$ofile)) {
        # Source'd via R console
        return(normalizePath(sys.frames()[[1]]$ofile))
      } else {
        tryCatch(
          {
            if (requireNamespace("knitr", quietly = TRUE)) {

              # Executing in a RMarkdown file
              pth <- knitr::current_input(dir = TRUE)
              if (length(pth) > 0 && nchar(pth) > 0) {
                return(normalizePath(pth))
              }
            }

            if (requireNamespace("rstudioapi", quietly = TRUE) &&
              rstudioapi::isAvailable()) {

              # RStudio Run Selection
              # http://stackoverflow.com/a/35842176/2292993
              pth <- rstudioapi::getActiveDocumentContext()$path
              if (length(pth) > 0 && nchar(pth) > 0) {
                return(normalizePath(pth))
              }

              # RStudio Console
              pth <- rstudioapi::getSourceEditorContext()$path
              if (length(pth) > 0 && nchar(pth) > 0) {
                return(normalizePath(pth))
              }

              # RStudio Source Editor
              pth <- rstudioapi::getConsoleEditorContext()$path
              if (length(pth) > 0 && nchar(pth) > 0) {
                return(normalizePath(pth))
              }
            }

            stop()
          },
          error = function(e) {
            if (fail == "stop") {
              stop("Unable to resolve file path.")
            } else if (fail == "warning") {
              warning("Unable to resolve file path, returning NA.")
              return(NA)
            }
            else { # fail=="quiet"
              return(NA)
            }
          }
        )
      }
    }
  }

  NA
}


#' @describeIn script_file  Determine the directory of the currently executing script
#'
#' @export
script_path <- function(fail = c("stop", "warning", "quiet")) {
  fail <- match.arg(fail)

  file <- script_file(fail = fail)
  if (is.na(file)) {
    NA
  } else {
    dirname(file)
  }
}

#' Convert between characters and ASCII codes
#'
#' @aliases asc chr
#' @param char vector of character strings
#' @param simplify logical indicating whether to attempt to convert the result
#' into a vector or matrix object. See \code{\link[base]{sapply}} for details.
#' @param ascii vector or list of vectors containing integer ASCII codes
#' @return \code{asc} returns the integer ASCII values for each character in
#' the elements of \code{char}.  If \code{simplify=FALSE} the result will be a
#' list containing one vector per element of \code{char}.  If
#' \code{simplify=TRUE}, the code will attempt to convert the result into a
#' vector or matrix.
#'
#' \code{asc} returns the characters corresponding to the provided ASCII
#' values.
#' @author Adapted by Gregory R. Warnes \email{greg@@warnes.net} from code
#' posted by Mark Davis on the 'Data Debrief' blog on 2011-03-09 at
#' \url{https://datadebrief.blogspot.com/2011/03/ascii-code-table-in-r.html}.
#' @seealso \code{\link[base]{strtoi}}, \code{\link[base]{charToRaw}},
#' \code{\link[base]{rawToChar}}, \code{\link[base]{as.raw}}
#' @keywords character programming
#' @examples
#'
#' ## ascii codes for lowercase letters
#' asc(letters)
#'
#' ## uppercase letters from ascii codes
#' chr(65:90)
#'
#' ## works on muti-character strings
#' (tmp <- asc("hello!"))
#' chr(tmp)
#'
#' ## Use 'simplify=FALSE' to return the result as a list
#' (tmp <- asc("hello!", simplify = FALSE))
#' chr(tmp)
#'
#' ## When simplify=FALSE the results can be...
#' asc(c("a", "e", "i", "o", "u", "y")) # a vector
#' asc(c("ae", "io", "uy")) # or a matrix
#'
#' ## When simplify=TRUE the results are always a list...
#' asc(c("a", "e", "i", "o", "u", "y"), simplify = FALSE)
#' asc(c("ae", "io", "uy"), simplify = FALSE)
#' @name asc
NULL

#' @describeIn asc return the characters corresponding to the specified ASCII codes
#' @export
asc <- function(char, simplify = TRUE) {
  sapply(char, function(x) strtoi(charToRaw(x), 16L), simplify = simplify)
}

#' @describeIn asc return the ASCII codes for the specified characters.
#' @export
chr <- function(ascii) sapply(ascii, function(x) rawToChar(as.raw(x)))

#' Display a prompt and collect the user's response
#'
#' Display a prompt and collect the user's response
#'
#' The prompt message will be displayed, and then \code{readLines} is used to
#' collect a single input value (possibly empty), which is then returned.
#'
#' In most situations using the default \code{con=stdin()} should work
#' properly.  Under RStudio, it is necessary to specify
#' \code{con=file("stdin")} for proper operation.
#'
#' @param msg Character vector providing the message to be displayed
#' @param con Character connection to query, defaults to \code{stdin()}.
#' @return A character scalar containing the input provided by the user.
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link{readLines}}, \code{\link{scan}}
#' @keywords IO
#' @examples
#'
#'
#' # use default prompt
#' ask()
#'
#' silly <- function() {
#'   age <- ask("How old aroe you? ")
#'   age <- as.numeric(age)
#'   cat("In 10 years you will be", age + 10, "years old!\n")
#' }
#' @export
ask <- function(msg = "Press <RETURN> to continue: ", con = stdin()) {
  cat(msg)
  readLines(con = con, n = 1)
}

newVersionAvailable <- function(quiet=FALSE)
  {
    page <- scan(file="https://cran.r-project.org/src/base/R-2", what="", quiet=TRUE)
    matches <- grep("R-[0-9]\\.[0-9]+\\.[0-9]+", page, value=TRUE)
    versionList <- gsub("^.*R-([0-9].[0-9]+.[0-9]+).*$","\\1",matches)
    versionList <- numeric_version(versionList)
    if( max(versionList) > getRversion() )
      {
        if(!quiet)
          {
            cat("A newer version of R is now available: ")
            cat(max(versionList))
            cat("\n")
          }
        invisible( max(versionList) )
     }
    else
      {
        if(!quiet)
          {
            cat("The latest version of R is installed: ")
            cat(as.character(getRversion()))
            cat("\n")
          }
        invisible( NULL );
      }

  }

library(gtools)

today <- Sys.Date()
tenweeks <- seq(today, length.out=10, by="1 week")

df1 <- data.frame(dates=tenweeks, chars=letters[1:10], ints=1:10, numeric=1.1:10.1)
df2 <- data.frame(chars=letters[11:20], ints=11:20, numeric=11.1:20.1)

smartbind(df1, df2)

#' Defunct Functions in package \code{gtools}
#'
#' The functions or variables listed here are no longer part of package
#' \code{gtools}.
#'
#' @param expression,collapse,x,\dots ignored
#'
#' @details
#'
#' \itemize{
#' \item \code{assert} is a defunct synonym for \code{\link[base]{stopifnot}}.
#' \item \code{addLast} has been replaced by \code{lastAdd}, which has the same
#'   purpose but applied using different syntax.
#' \item \code{capture} and \code{capture.output} have been removed in favor of
#'   \code{capture.output} from the \code{utils} package.
#' }
#'
#' @aliases gtools-defunct assert addLast capture sprint
#'
#' @seealso
#'   \code{\link[base]{Defunct}},
#'   \code{\link[base]{stopifnot}},
#'   \code{\link[gtools]{lastAdd}},
#'   \code{\link[utils]{capture.output}}
#' @keywords misc
#'
#' @rdname gtools-defunct
#' @export
assert <- function(...) {
  .Defunct("stopifnot", "base")
}
#'
#' @rdname gtools-defunct
#' @export
capture <- function(expression, collapse = "\n") {
  .Defunct("capture.output", "base")
}

#' @rdname gtools-defunct
#' @export
sprint <- function(x, ...) {
  .Defunct("capture.output", "base")
}

#' Replace Missing Values
#'
#' Replace missing values
#'
#' This is a convenience function that is the same as x[is.na(x)] <- replace
#'
#' @param x vector possibly containing missing (\code{NA}) values
#' @param replace either a scalar replacement value, or a function returning a
#' scalar value
#' @param ... Optional arguments to be passed to \code{replace}
#' @return Vector with missing values (\code{NA}) replaced by the value of
#' \code{replace}.
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link[base]{is.na}}, \code{\link[stats]{na.omit}}
#' @keywords manip
#' @examples
#'
#' x <- c(1, 2, 3, NA, 6, 7, 8, NA, NA)
#'
#' # Replace with a specified value
#' na.replace(x, "999")
#'
#' # Replace with the calculated median
#' na.replace(x, median, na.rm = TRUE)
#' @export
na.replace <- function(x, replace, ...) {
  if (is.function(replace)) {
    replace <- replace(x, ...)
  }

  x[is.na(x)] <- replace
  x
}

#' Enumerate the Combinations or Permutations of the Elements of a Vector
#' 
#' \code{combinations} enumerates the possible combinations of a specified size
#' from the elements of a vector.  \code{permutations} enumerates the possible
#' permutations.
#' 
#' Caution: The number of combinations and permutations increases rapidly with
#' \code{n} and \code{r}!.
#' 
#' To use values of \code{n} above about 45, you will need to increase R's
#' recursion limit.  See the \code{expression} argument to the \code{options}
#' command for details on how to do this.
#' 
#' @aliases combinations permutations
#' @param n Size of the source vector
#' @param r Size of the target vectors
#' @param v Source vector. Defaults to \code{1:n}
#' @param set Logical flag indicating whether duplicates should be removed from
#' the source vector \code{v}. Defaults to \code{TRUE}.
#' @param repeats.allowed Logical flag indicating whether the constructed
#' vectors may include duplicated values.  Defaults to \code{FALSE}.
#' @return Returns a matrix where each row contains a vector of length
#' \code{r}.
#' @author Original versions by Bill Venables
#' \email{Bill.Venables@@cmis.csiro.au}.  Extended to handle
#' \code{repeats.allowed} by Gregory R. Warnes \email{greg@@warnes.net}.
#' @seealso \code{\link[base]{choose}}, \code{\link[base]{options}}
#' @references Venables, Bill.  "Programmers Note", R-News, Vol 1/1, Jan. 2001.
#' \url{https://cran.r-project.org/doc/Rnews/}
#' @keywords manip
#' 
#' @details 
#' 
#' Taken from an email by Brian D Ripley <ripley@stats.ox.ac.uk> to r-help
#' dated Tue, 14 Dec 1999 11:14:04 +0000 (GMT) in response to
#' Alex Ahgarin \email{datamanagement@email.com}.  Original version was
#' named "subsets" and was Written by Bill Venables.  
#'
#' @examples
#' 
#' combinations(3,2,letters[1:3])
#' combinations(3,2,letters[1:3],repeats=TRUE)
#' 
#' permutations(3,2,letters[1:3])
#' permutations(3,2,letters[1:3],repeats=TRUE)
#' 
#' \dontrun{
#' # To use large 'n', you need to change the default recusion limit
#' options(expressions=1e5)
#' cmat <- combinations(300,2)
#' dim(cmat) # 44850 by 2 
#' }
#' 
#' @export
combinations <- function(n, r, v = 1:n, set = TRUE, repeats.allowed=FALSE)
{
  if(mode(n) != "numeric" || length(n) != 1 
     || n < 1 || (n %% 1) != 0) stop("bad value of n") 
  if(mode(r) != "numeric" || length(r) != 1 
     || r < 1 || (r %% 1) != 0) stop("bad value of r") 
  if(!is.atomic(v) || length(v) < n) 
    stop("v is either non-atomic or too short")
  if( (r > n) & repeats.allowed==FALSE)
    stop("r > n and repeats.allowed=FALSE")
  if(set) {
    v <- unique(sort(v))
    if (length(v) < n) stop("too few different elements")
  }
  v0 <- vector(mode(v), 0)
  ## Inner workhorse
  if(repeats.allowed)
    sub <- function(n, r, v)
      { 
        if(r == 0) v0 else
        if(r == 1) matrix(v, n, 1) else
        if(n == 1) matrix(v, 1, r) else
        rbind( cbind(v[1], Recall(n, r-1, v)),
              Recall(n-1, r, v[-1]))
      }
  else
    sub <- function(n, r, v)
      { 
        if(r == 0) v0 else
        if(r == 1) matrix(v, n, 1) else
        if(r == n) matrix(v, 1, n) else
        rbind(cbind(v[1], Recall(n-1, r-1, v[-1])),
              Recall(n-1, r, v[-1]))
      }
  sub(n, r, v[1:n])
}

##
## Original version by Bill Venables and cited by by Matthew
## Wiener (mcw@ln.nimh.nih.gov) in an email to R-help dated
## Tue, 14 Dec 1999 09:11:32 -0500 (EST) in response to
## Alex Ahgarin <datamanagement@email.com>
##
##

#' @rdname combinations
#' @export
permutations <- function(n, r, v = 1:n, set = TRUE, repeats.allowed=FALSE)
{
  if(mode(n) != "numeric" || length(n) != 1 
     || n < 1 || (n %% 1) != 0) stop("bad value of n") 
  if(mode(r) != "numeric" || length(r) != 1 
     || r < 1 || (r %% 1) != 0) stop("bad value of r") 
  if(!is.atomic(v) || length(v) < n) 
    stop("v is either non-atomic or too short")
  if( (r > n) & repeats.allowed==FALSE)
    stop("r > n and repeats.allowed=FALSE")
  if(set) {
    v <- unique(sort(v))
    if (length(v) < n) stop("too few different elements")
  }
  v0 <- vector(mode(v), 0)
  ## Inner workhorse
  if(repeats.allowed)
    sub <- function(n, r, v)
      {
        if(r==1) matrix(v,n,1) else
        if(n==1) matrix(v,1,r) else
        {
          inner  <-  Recall(n, r-1, v)
          cbind( rep( v, rep(nrow(inner),n)  ),
                 matrix( t(inner), ncol=ncol(inner), nrow=nrow(inner) * n ,
                        byrow=TRUE )
                )
        }
      }
  else
    sub <- function(n, r, v)
      {
        if(r==1) matrix(v,n,1) else
        if(n==1) matrix(v,1,r) else
        {
        X  <-  NULL
        for(i in 1:n)
          X  <-  rbind( X, cbind( v[i], Recall(n-1, r - 1, v[-i])))
        X
        }
      }

  sub(n, r, v[1:n])
}

#' Split a File Path into Components
#'
#' @description This function converts a character scalar containing a
#'  \emph{valid} file path into a character vector of path components
#'  (e.g. directories).
#'
#' @param x            character scalar.  Path to be processed.
#' @param depth_first  logical.  Should path be returned depth first?  Defaults
#'   to \code{TRUE}.
#'
#' @return Character vector of path components, depth first.
#'
#' @export
#'
split_path <- function(x, depth_first=TRUE)
{
  if(length(x)>1) warning("This function is not vectorized.",
                          "Only processing the first element of x.")
  retval <- split_path_inner(x)
  if(!depth_first)
    retval <- rev(retval)
  retval[retval>""]
}


split_path_inner <- function(path) {
  if (dirname(path) %in% c(".", path)) return(basename(path))
  return(c(basename(path), split_path_inner(dirname(path))))
}

#' Generate significance stars from p-values
#'
#' Generate significance stars (e.g. '***', '**', '*', '.') from p-values using
#' R's standard definitions.
#'
#' Mapping from p-value ranges to symbols: \describe{ \item{0 - 0.001}{'***'}
#' \item{0.001 - 0.01}{'**'} \item{0.01 - 0.05}{'*'} \item{0.05 - 0.1}{'.'}
#' \item{0.1 - 1.0}{'' (No symbol)} }
#'
#' @param p.value numeric vector of p-values
#' @return A character vector containing the same number of elements as
#' \code{p-value}, with an attribute "legend" providing the conversion pattern.
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link[stats]{symnum}}
#' @keywords misc
#' @examples
#'
#' p.val <- c(0.0004, 0.0015, 0.013, 0.044, 0.067, 0.24)
#' stars.pval(p.val)
#' @importFrom stats symnum
#' @export
stars.pval <- function(p.value) {
  unclass(
    symnum(p.value,
      corr = FALSE, na = FALSE,
      cutpoints = c(0, 0.001, 0.01, 0.05, 0.1, 1),
      symbols = c("***", "**", "*", ".", " ")
    )
  )
}

packagefile="gdata_2.16.0.tar.gz"
destdir=tempdir()

checkReverseDependencies <- function(packagefile, destdir=tempdir(), cleanup=FALSE )
    {
        if(!file.exists(packagefile))
            stop(packagefile, " does not exist!")

        cat("Using directory '", destdir, "'.  Remember to delete it when done.\n", sep='')

        file.copy(packagefile, destdir)

        package <- gsub("_.*$", "", packagefile)

        rdeps <- tools::package_dependencies(package, db=available.packages(), reverse = TRUE)[[1]]
        cat( length(rdeps), "reverse dependencies:\n")
        print(rdeps)

        tools::check_packages_in_dir(destdir, reverse=list(), Ncpus=6)

        if(cleanup) unlink(destdir, recursive=TRUE, force=TRUE)
    }

#' Detect odd/even integers
#'
#' detect odd/even integers
#'
#'
#' @aliases odd even
#' @param x vector of integers
#' @return Vector of TRUE/FALSE values.
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link[base]{round}}
#' @keywords arith
#' @examples
#'
#'
#' odd(4)
#' even(4)
#'
#' odd(1:10)
#' even(1:10)
#' @name oddeven
NULL

#' @rdname oddeven
#' @export
odd <- function(x) x %% 2 == 1

#' @rdname oddeven
#' @export
even <- function(x) x %% 2 == 0

#' Deprecated Functions in the gtools package
#'
#' These functions are provided for compatibility with older versions of
#' gtools, and may be defunct as soon as the next release.
#'
#' gtools currently contains no deprecated functions.
#'
#' % The original help page for these functions is often % available at
#' \code{help("oldName-deprecated")} (note the quotes). % % \itemize{ % \item{}
#' % }
#'
#' @seealso \code{\link{Deprecated}}
#' @keywords misc
#' @name gtools-deprecated
NULL

#' Apply a Function Over Adjacent Subsets of a Vector
#'
#' Applies a function over subsets of the vector(s) formed by taking a fixed
#' number of previous points.
#'
#' \code{running} applies the specified function to a sequential windows on
#' \code{X} and (optionally) \code{Y}.  If \code{Y} is specified the function
#' must be bivariate.
#'
#' @param X data vector
#' @param Y data vector (optional)
#' @param fun Function to apply. Default is \code{mean}
#' @param width Integer giving the number of vector elements to include in the
#' subsets.  Defaults to the lesser of the length of the data and 20 elements.
#' @param allow.fewer Boolean indicating whether the function should be
#' computed for subsets with fewer than \code{width} points
#' @param pad Boolean indicating whether the returned results should be
#' 'padded' with NAs corresponding to sets with less than \code{width}
#' elements.  This only applies when when \code{allow.fewer} is FALSE.
#' @param align One of "right", "center", or "left".  This controls the
#' relative location of `short' subsets with less then \code{width} elements:
#' "right" allows short subsets only at the beginning of the sequence so that
#' all of the complete subsets are at the end of the sequence (i.e. `right
#' aligned'), "left" allows short subsets only at the end of the data so that
#' the complete subsets are `left aligned', and "center" allows short subsets
#' at both ends of the data so that complete subsets are `centered'.
#' @param simplify Boolean.  If FALSE the returned object will be a list
#' containing one element per evaluation.  If TRUE, the returned object will be
#' coerced into a vector (if the computation returns a scalar) or a matrix (if
#' the computation returns multiple values).  Defaults to FALSE.
#' @param by Integer separation between groups. If \code{by=width} will give
#' non-overlapping windows. Default is missing, in which case groups will start
#' at each value in the X/Y range.
#' @param \dots parameters to be passed to \code{fun}
#' @return List (if \code{simplify==TRUE}), vector, or matrix containing the
#' results of applying the function \code{fun} to the subsets of \code{X}
#' (\code{running}) or \code{X} and \code{Y}.
#'
#' Note that this function will create a vector or matrix even for objects
#' which are not simplified by \code{sapply}.
#' @author Gregory R. Warnes \email{greg@@warnes.net}, with contributions by
#' Nitin Jain \email{nitin.jain@@pfizer.com}.
#' @seealso \code{\link[gplots]{wapply}} to apply a function over an x-y window
#' centered at each x point, \code{\link[base]{sapply}},
#' \code{\link[base]{lapply}}
#' @keywords misc
#' @examples
#'
#'
#' # show effect of pad
#' running(1:20, width = 5)
#' running(1:20, width = 5, pad = TRUE)
#'
#' # show effect of align
#' running(1:20, width = 5, align = "left", pad = TRUE)
#' running(1:20, width = 5, align = "center", pad = TRUE)
#' running(1:20, width = 5, align = "right", pad = TRUE)
#'
#' # show effect of simplify
#' running(1:20, width = 5, fun = function(x) x) # matrix
#' running(1:20, width = 5, fun = function(x) x, simplify = FALSE) # list
#'
#' # show effect of by
#' running(1:20, width = 5) # normal
#' running(1:20, width = 5, by = 5) # non-overlapping
#' running(1:20, width = 5, by = 2) # starting every 2nd
#'
#'
#' # Use 'pad' to ensure correct length of vector, also show the effect
#' # of allow.fewer.
#' par(mfrow = c(2, 1))
#' plot(1:20, running(1:20, width = 5, allow.fewer = FALSE, pad = TRUE), type = "b")
#' plot(1:20, running(1:20, width = 5, allow.fewer = TRUE, pad = TRUE), type = "b")
#' par(mfrow = c(1, 1))
#'
#' # plot running mean and central 2 standard deviation range
#' # estimated by *last* 40 observations
#' dat <- rnorm(500, sd = 1 + (1:500) / 500)
#' plot(dat)
#' sdfun <- function(x, sign = 1) mean(x) + sign * sqrt(var(x))
#' lines(running(dat, width = 51, pad = TRUE, fun = mean), col = "blue")
#' lines(running(dat, width = 51, pad = TRUE, fun = sdfun, sign = -1), col = "red")
#' lines(running(dat, width = 51, pad = TRUE, fun = sdfun, sign = 1), col = "red")
#'
#'
#' # plot running correlation estimated by last 40 observations (red)
#' # against the true local correlation (blue)
#' sd.Y <- seq(0, 1, length = 500)
#'
#' X <- rnorm(500, sd = 1)
#' Y <- rnorm(500, sd = sd.Y)
#'
#' plot(running(X, X + Y, width = 20, fun = cor, pad = TRUE), col = "red", type = "s")
#'
#' r <- 1 / sqrt(1 + sd.Y^2) # true cor of (X,X+Y)
#' lines(r, type = "l", col = "blue")
#' @importFrom methods new
#' @export
running <- function(X, Y = NULL,
                    fun = mean,
                    width = min(length(X), 20),
                    allow.fewer = FALSE, pad = FALSE,
                    align = c("right", "center", "left"),
                    simplify = TRUE,
                    by, # added a parameter
                    ...) {
  align <- match.arg(align)

  n <- length(X)

  if (align == "left") {
    from <- 1:n
    to <- pmin((1:n) + width - 1, n)
  }
  else if (align == "right") {
    from <- pmax((1:n) - width + 1, 1)
    to <- 1:n
  }
  else # align=="center"
  {
    from <- pmax((2 - width):n, 1)
    to <- pmin(1:(n + width - 1), n)
    if (!odd(width)) stop("width must be odd for center alignment")
  }

  elements <- apply(cbind(from, to), 1, function(x) seq(x[1], x[2]))

  if (is.matrix(elements)) {
    elements <- as.data.frame(elements)
  } # ensure its a list!

  names(elements) <- paste(from, to, sep = ":")

  if (!allow.fewer) {
    len <- sapply(elements, length)
    skip <- (len < width)
  }
  else {
    skip <- 0
  }


  run.elements <- elements[!skip]

  if (!invalid(by)) {
    run.elements <- run.elements[seq(
      from = 1, to = length(run.elements),
      by = by
    )]
  }


  if (is.null(Y)) # univariate
    {
      funct.uni <- function(which, what, fun, ...) fun(what[which], ...)

      if (simplify) {
        Xvar <- sapply(run.elements, funct.uni, what = X, fun = fun, ...)
      } else {
        Xvar <- lapply(run.elements, funct.uni, what = X, fun = fun, ...)
      }
    }
  else # bivariate
  {
    funct.bi <- function(which, XX, YY, fun, ...) fun(XX[which], YY[which], ...)

    if (simplify) {
      Xvar <- sapply(run.elements, funct.bi, XX = X, YY = Y, fun = fun, ...)
    } else {
      Xvar <- lapply(run.elements, funct.bi, XX = X, YY = Y, fun = fun, ...)
    }
  }


  if (allow.fewer || !pad) {
    return(Xvar)
  }

  if (simplify) {
    if (is.matrix(Xvar)) {
      wholemat <- matrix(new(class(Xvar[1, 1]), NA),
        ncol = length(to), nrow = nrow(Xvar)
      )
      colnames(wholemat) <- paste(from, to, sep = ":")
      wholemat[, -skip] <- Xvar
      Xvar <- wholemat
    }
    else {
      wholelist <- rep(new(class(Xvar[1]), NA), length(from))
      names(wholelist) <- names(elements)
      wholelist[names(Xvar)] <- Xvar
      Xvar <- wholelist
    }
  }

  return(Xvar)
}

## Regression test for bug reported by John Nolan:
##
## Whenever the pair (x[i], a[i]) == (0,1), NA would be returned, due
## to an internal computation of ( 0 * -Inf ) => NaN
##
## The code now checks for this particular issue and sets the value of
## ( 0 * -Inf ) to 0, which is correct for this calculation.
##
library(gtools)

x = c(0,0,1)
alpha = c(1,2,3)

stopifnot( ddirichlet(x=x, alpha=alpha) == 0 )

#' Display debugging text
#'
#' If \code{getOption('DEBUG')==TRUE}, write text to STDOUT and flush so that
#' the text is immediately displayed. Otherwise, do nothing.
#'
#'
#' @param \dots Arguments passed to \code{cat}
#' @return NULL (invisibly)
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link[base]{cat}}
#' @keywords print
#' @examples
#'
#' options(DEBUG = NULL) # makee sure DEBUG isn't set
#' scat("Not displayed")
#'
#' options(DEBUG = TRUE)
#' scat("This will be displayed immediately (even in R BATCH output \n")
#' scat("files), provided options()$DEBUG is TRUE.")
#' @export
scat <- function(...) {
  DEBUG <- options()$DEBUG
  if (!is.null(DEBUG) && DEBUG) {
    cat("### ", file = stderr())
    cat(..., file = stderr())
    cat(" ###\n", file = stderr())
    flush(stderr())
  }
  invisible(NULL)
}

#' Define a macro
#'
#' \code{defmacro} define a macro that uses R expression replacement
#'
#' \code{strmacro} define a macro that uses string replacement
#'
#'
#' \code{defmacro} and \code{strmacro} create a macro from the expression given
#' in \code{expr}, with formal arguments given by the other elements of the
#' argument list.
#'
#' A macro is similar to a function definition except for handling of formal
#' arguments.  In a function, formal arguments are simply variables that
#' contains the result of evaluating the expressions provided to the function
#' call.  In contrast, macros actually modify the macro body by
#' \emph{replacing} each formal argument by the expression (\code{defmacro}) or
#' string (\code{strmacro}) provided to the macro call.
#'
#' For \code{defmacro}, the special argument name \code{DOTS} will be replaced
#' by \code{...} in the formal argument list of the macro so that \code{...} in
#' the body of the expression can be used to obtain any additional arguments
#' passed to the macro. For \code{strmacro} you can mimic this behavior
#' providing a \code{DOTS=""} argument.  This is illustrated by the last
#' example below.
#'
#' Macros are often useful for creating new functions during code execution.
#'
#' @aliases defmacro strmacro
#' @param \dots macro argument list
#' @param expr R expression defining the macro body
#' @param strexpr character string defining the macro body
#' @return A macro function.
#'
#' @note Note that because [the defmacro code] works on the parsed expression,
#' not on a text string, defmacro avoids some of the problems of traditional
#' string substitution macros such as \code{strmacro} and the C preprocessor
#' macros. For example, in \preformatted{ mul <- defmacro(a, b, expr={a*b}) } a
#' C programmer might expect \code{mul(i, j + k)} to expand (incorrectly) to
#' \code{i*j + k}. In fact it expands correctly, to the equivalent of
#' \code{i*(j + k)}.
#'
#' For a discussion of the differences between functions and macros, please
#' Thomas Lumley's R-News article (reference below).
#' @author Thomas Lumley wrote \code{defmacro}.  Gregory R. Warnes
#' \email{greg@@warnes.net} enhanced it and created \code{strmacro}.
#' @seealso \code{\link[base]{function}} \code{\link[base]{substitute}},
#' \code{\link[base]{eval}}, \code{\link[base]{parse}},
#' \code{\link[base]{source}}, \code{\link[base]{parse}},
#' @references The original \code{defmacro} code was directly taken from:
#'
#' Lumley T. "Programmer's Niche: Macros in R", R News, 2001, Vol 1, No. 3, pp
#' 11--13, \url{https://cran.r-project.org/doc/Rnews/}
#' @keywords programming
#' ## Code from
#' @examples
#'
#' ####
#' # macro for replacing a specified missing value indicator with NA
#' # within a dataframe
#' ###
#' setNA <- defmacro(df, var, values,
#'   expr = {
#'     df$var[df$var %in% values] <- NA
#'   }
#' )
#'
#' # create example data using 999 as a missing value indicator
#' d <- data.frame(
#'   Grp = c("Trt", "Ctl", "Ctl", "Trt", "Ctl", "Ctl", "Trt", "Ctl", "Trt", "Ctl"),
#'   V1 = c(1, 2, 3, 4, 5, 6, 999, 8, 9, 10),
#'   V2 = c(1, 1, 1, 1, 1, 2, 999, 2, 999, 999),
#'   stringsAsFactors = TRUE
#' )
#' d
#'
#' # Try it out
#' setNA(d, V1, 999)
#' setNA(d, V2, 999)
#' d
#'
#'
#' ###
#' # Expression macro
#' ###
#' plot.d <- defmacro(df, var, DOTS,
#'   col = "red", title = "", expr =
#'     plot(df$var ~ df$Grp, type = "b", col = col, main = title, ...)
#' )
#'
#' plot.d(d, V1)
#' plot.d(d, V1, col = "blue")
#' plot.d(d, V1, lwd = 4) # use optional 'DOTS' argument
#'
#' ###
#' # String macro (note the quoted text in the calls below)
#' #
#' # This style of macro can be useful when you are reading
#' # function arguments from a text file
#' ###
#' plot.s <- strmacro(DF, VAR,
#'   COL = "'red'", TITLE = "''", DOTS = "", expr =
#'     plot(DF$VAR ~ DF$Grp, type = "b", col = COL, main = TITLE, DOTS)
#' )
#'
#' plot.s("d", "V1")
#' plot.s(DF = "d", VAR = "V1", COL = '"blue"')
#' plot.s("d", "V1", DOTS = "lwd=4") # use optional 'DOTS' argument
#'
#'
#'
#' #######
#' # Create a macro that defines new functions
#' ######
#' plot.sf <- defmacro(
#'   type = "b", col = "black",
#'   title = deparse(substitute(x)), DOTS, expr =
#'     function(x, y) plot(x, y, type = type, col = col, main = title, ...)
#' )
#'
#' plot.red <- plot.sf(col = "red", title = "Red is more Fun!")
#' plot.blue <- plot.sf(col = "blue", title = "Blue is Best!", lty = 2)
#'
#' plot.red(1:100, rnorm(100))
#' plot.blue(1:100, rnorm(100))
#' @export
defmacro <- function(..., expr) # , DOTS=FALSE)
{
  expr <- substitute(expr)
  a <- substitute(list(...))[-1]

  ## process the argument list
  nn <- names(a)
  if (is.null(nn)) {
    nn <- rep("", length(a))
  }
  for (i in 1:length(a))
  {
    if (nn[i] == "") {
      nn[i] <- paste(a[[i]])
      msg <- paste(a[[i]], "not supplied")
      a[[i]] <- substitute(
        stop(foo),
        list(foo = msg)
      )
    }
    if (nn[i] == "DOTS") {
      nn[i] <- "..."
      a[[i]] <- formals(function(...) {})[[1]]
    }
  }
  names(a) <- nn
  a <- as.list(a)

  ## this is where the work is done
  ff <- eval(substitute(
    function() {
      tmp <- substitute(body)
      eval(tmp, parent.frame())
    },
    list(body = expr)
  ))

  ## add the argument list
  formals(ff) <- a

  ## create a fake source attribute
  mm <- match.call()
  mm$expr <- NULL
  mm[[1]] <- as.name("macro")
  attr(ff, "source") <- c(
    deparse(mm),
    deparse(expr)
  )

  ## return the 'macro'
  ff
}

#' Get package dependencies
#'
#' Get package dependencies
#'
#' This function recursively constructs the list of dependencies for the
#' packages given by \code{pkgs}.  By default, the dependency information is
#' extracted from both installed and available packages.  As a consequence, it
#' works both for local and CRAN packages.
#'
#' @param pkgs character vector of package names
#' @param dependencies character vector of dependency types to include.
#' Choices are "Depends", "Imports", "LinkingTo", "Suggests", and "Enhances".
#' Defaults to \code{c("Depends", "Imports", "LinkingTo")}.
#' @param installed Logical indicating whether to pull dependency information
#' from installed packages. Defaults to TRUE.
#' @param available Logical indicating whether to pull dependency information
#' from available packages.  Defaults to TRUE.
#' @param base Logical indicating whether to include dependencies on base
#' packages that are included in the R installation. Defaults to FALSE.
#' @param recommended Logical indicating whether to include dependencies on
#' recommended packages that are included in the R installation. Defaults to
#' FALSE.
#' @return A character vector of package names.
#' @note If \code{available=TRUE} R will attempt to access the currently
#' selected CRAN repository, prompting for one if necessary.
#' @author Gregory R. Warnes \email{greg@@warnes.net} based on the non exported
#' \code{utils:::getDependencies} and \code{utils:::.clean_up_dependencies2}.
#' @seealso \code{\link{installed.packages}}, \code{\link{available.packages}}
#' @keywords utilities
#' @examples
#'
#' \dontrun{
#' ## A locally installed package
#' #' getDependencies("MASS", installed = TRUE, available = FALSE)
#' 
#' ## A package on CRAN
#' getDependencies("gregmisc", installed = FALSE, available = TRUE)
#' 
#'
#' ## Show base and recommended dependencies
#' getDependencies("MASS", available = FALSE, base = TRUE, recommended = TRUE)
#' 
#' ## Download the set of packages necessary to support a local package
#' deps <- getDependencies("MyLocalPackage", available = FALSE)
#' download.packages(deps, destdir = "./R_Packages")
#' }
#'
#' @importFrom utils available.packages
#' @importFrom utils flush.console
#' @importFrom utils head
#' @importFrom utils installed.packages
#' @export
getDependencies <- function(pkgs,
                            dependencies = c("Depends", "Imports", "LinkingTo"),
                            installed = TRUE,
                            available = TRUE,
                            base = FALSE,
                            recommended = FALSE) {
  pkgs.in <- pkgs
  if (is.null(dependencies)) {
    return(unique(pkgs))
  }
  dep2 <- c("Depends", "Imports", "LinkingTo")


  if (installed && !available) {
    all.packages <- installed.packages()
  } else if (available && !installed) {
    all.packages <- available.packages()
  } else {
    all.packages <- as.matrix(smartbind( ##
      installed.packages(),
      available.packages()
    ))
  }
  rownames(all.packages) <- all.packages[, "Package"]

  p0 <- unique(pkgs)
  miss <- !p0 %in% row.names(all.packages)
  if (sum(miss)) {
    warning(sprintf(ngettext(
      sum(miss), "package %s is not available (for %s)",
      "packages %s are not available (for %s)"
    ), paste(sQuote(p0[miss]),
      collapse = ", "
    ), sub(" *\\(.*", "", R.version.string)),
    domain = NA, call. = FALSE
    )
    if (sum(miss) == 1L && !is.na(w <- match(
      tolower(p0[miss]),
      tolower(row.names(all.packages))
    ))) {
      warning(sprintf("Perhaps you meant %s ?", sQuote(row.names(all.packages)[w])),
        call. = FALSE, domain = NA
      )
    }
    flush.console()
  }

  ## Whether to exclude base and recommended packages
  if (!base || !recommended) {
    priority <- NULL
    if (!base) priority <- c("base", priority)
    if (!recommended) priority <- c("recommended", priority)
    installed <- installed.packages(priority = priority)
  }
  else {
    installed <- installed.packages()[FALSE, ]
  }

  p0 <- p0[!miss]

  p1 <- p0
  not_avail <- character()
  repeat {
    deps <- apply(
      all.packages[p1, dependencies, drop = FALSE],
      1L, function(x) paste(x[!is.na(x)], collapse = ", ")
    )

    res <- .clean_up_dependencies2(
      deps,
      installed = installed,
      all.packages
    )
    not_avail <- c(not_avail, res[[2L]])
    deps <- unique(res[[1L]])
    deps <- deps[!deps %in% c("R", pkgs)]
    if (!length(deps)) {
      break
    }
    pkgs <- c(deps, pkgs)
    p1 <- deps
    if (!is.null(dep2)) {
      dependencies <- dep2
      dep2 <- NULL
    }
  }
  if (length(not_avail)) {
    not_avail <- unique(not_avail)
    warning(sprintf(
      ngettext(
        length(not_avail),
        "dependency %s is not available",
        "dependencies %s are not available"
      ),
      paste(sQuote(not_avail),
        collapse = ", "
      )
    ),
    domain = NA,
    call. = FALSE,
    immediate. = TRUE
    )
    flush.console()
  }
  pkgs <- unique(pkgs)
  pkgs <- pkgs[pkgs %in% row.names(all.packages)]
  p0 <- pkgs


  p0[!p0 %in% pkgs.in]
}

.clean_up_dependencies2 <- function(x, installed, available) {
  .split_op_version <- function(x) {
    pat <- "^([^\\([:space:]]+)[[:space:]]*\\(([^\\)]+)\\).*"
    x1 <- sub(pat, "\\1", x)
    x2 <- sub(pat, "\\2", x)
    if (x2 != x1) {
      pat <- "[[:space:]]*([[<>=!]+)[[:space:]]+(.*)"
      version <- sub(pat, "\\2", x2)
      if (!grepl("^r", version)) {
        version <- package_version(version)
      }
      list(name = x1, op = sub(pat, "\\1", x2), version = version)
    }
    else {
      list(name = x1)
    }
  }

  .split_dependencies <- function(x) {
    .split2 <- function(x) {
      x <- sub("[[:space:]]+$", "", x)
      x <- unique(sub("^[[:space:]]*(.*)", "\\1", x))
      names(x) <- sub("^([[:alnum:].]+).*$", "\\1", x)
      x <- x[names(x) != "R"]
      x <- x[nzchar(x)]
      x <- x[!duplicated(names(x))]
      lapply(x, .split_op_version)
    }
    if (!any(nzchar(x))) {
      return(list())
    }
    unlist(lapply(strsplit(x, ","), .split2), FALSE, FALSE)
  }
  x <- x[!is.na(x)]
  if (!length(x)) {
    return(list(character(), character()))
  }
  xx <- .split_dependencies(x)
  if (!length(xx)) {
    return(list(character(), character()))
  }
  pkgs <- installed[, "Package"]
  have <- sapply(xx, function(x) {
    if (length(x) == 3L) {
      if (!x[[1L]] %in% pkgs) {
        return(FALSE)
      }
      if (x[[2L]] != ">=") {
        return(TRUE)
      }
      current <- as.package_version(installed[
        pkgs == x[[1L]],
        "Version"
      ])
      target <- as.package_version(x[[3L]])
      eval(parse(text = paste("any(current", x$op, "target)")))
    }
    else {
      x[[1L]] %in% pkgs
    }
  })
  xx <- xx[!have]
  if (!length(xx)) {
    return(list(character(), character()))
  }
  pkgs <- row.names(available)
  canget <- miss <- character()
  for (i in seq_along(xx)) {
    x <- xx[[i]]
    if (length(x) == 3L) {
      if (!x[[1L]] %in% pkgs) {
        miss <- c(miss, x[[1L]])
        next
      }
      if (x[[2L]] != ">=") {
        canget <- c(canget, x[[1L]])
        next
      }
      current <- as.package_version(available[
        pkgs == x[[1L]],
        "Version"
      ])
      target <- as.package_version(x[[3L]])
      res <- eval(parse(text = paste(
        "any(current", x$op,
        "target)"
      )))
      if (res) {
        canget <- c(canget, x[[1L]])
      } else {
        miss <- c(miss, paste0(
          x[[1L]], " (>= ", x[[3L]],
          ")"
        ))
      }
    }
    else if (x[[1L]] %in% pkgs) {
      canget <- c(canget, x[[1L]])
    } else {
      miss <- c(miss, x[[1L]])
    }
  }
  list(canget, miss)
}

#' Create a Factor Variable Using the Quantiles of a Continuous Variable
#'
#' Create a factor variable using the quantiles of a continuous variable.
#'
#'
#' This function uses \code{\link{quantile}} to obtain the specified quantiles
#' of \code{x}, then calls \code{\link{cut}} to create a factor variable using
#' the intervals specified by these quantiles.
#'
#' It properly handles cases where more than one quantile obtains the same
#' value, as in the second example below.  Note that in this case, there will
#' be fewer generated factor levels than the specified number of quantile
#' intervals.
#'
#' @param x Continuous variable.
#' @param q Either a integer number of equally spaced quantile groups to
#' create, or a vector of quantiles used for creating groups. Defaults to
#' \code{q=4} which is equivalent to \code{q=seq(0, 1, by=0.25)}.  See
#' \code{\link{quantile}} for details.
#' @param na.rm Boolean indicating whether missing values should be removed
#' when computing quantiles.  Defaults to TRUE.
#' @param \dots Optional arguments passed to \code{\link{cut}}.
#' @return Factor variable with one level for each quantile interval.
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link{cut}}, \code{\link{quantile}}
#' @keywords manip
#' @examples
#'
#'
#' ## create example data
#' # testonly{
#' set.seed(1234)
#' # }
#' x <- rnorm(1000)
#'
#' ## cut into quartiles
#' quartiles <- quantcut(x)
#' table(quartiles)
#'
#' ## cut into deciles
#' deciles.1 <- quantcut(x, 10)
#' table(deciles.1)
#' # or equivalently
#' deciles.2 <- quantcut(x, seq(0, 1, by = 0.1))
#'
#' # testonly{
#' stopifnot(identical(deciles.1, deciles.2))
#' # }
#'
#' ## show handling of 'tied' quantiles.
#' x <- round(x) # discretize to create ties
#' stem(x) # display the ties
#' deciles <- quantcut(x, 10)
#'
#' table(deciles) # note that there are only 5 groups (not 10)
#' # due to duplicates
#' @importFrom stats quantile
#' @export
quantcut <- function(x, q = 4, na.rm = TRUE, ...) {
  if (length(q) == 1) {
    q <- seq(0, 1, length.out = q + 1)
  }

  quant <- quantile(x, q, na.rm = na.rm)
  dups <- duplicated(quant)
  if (any(dups)) {
    flag <- x %in% unique(quant[dups])
    retval <- ifelse(flag,
      paste("[",
        as.character(x),
        "]",
        sep = ""
      ),
      NA
    )
    uniqs <- unique(quant)

    # move cut points over a bit...
    reposition <- function(cut) {
      flag <- x >= cut
      if (sum(flag, na.rm = na.rm) == 0) {
        return(cut)
      } else {
        return(min(x[flag], na.rm = na.rm))
      }
    }

    newquant <- sapply(uniqs, reposition)
    retval[!flag] <- as.character(cut(x[!flag],
      breaks = newquant,
      include.lowest = TRUE, ...
    ))

    levs <- unique(retval[order(x)]) # ensure factor levels are
    # properly ordered
    retval <- factor(retval, levels = levs)

    ## determine open/closed interval ends
    mkpairs <- function(x) { # make table of lower, upper
      sapply(
        x,
        function(y) if (length(y) == 2) y[c(2, 2)] else y[2:3]
      )
    }
    pairs <- mkpairs(strsplit(levs, "[^0-9+\\.\\-]+"))
    rownames(pairs) <- c("lower.bound", "upper.bound")
    colnames(pairs) <- levs

    closed.lower <- rep(F, ncol(pairs)) # default lower is open
    closed.upper <- rep(T, ncol(pairs)) # default upper is closed
    closed.lower[1] <- TRUE # lowest interval is always closed

    for (i in 2:ncol(pairs)) { # open lower interval if above singlet
      if (pairs[1, i] == pairs[1, i - 1] && pairs[1, i] == pairs[2, i - 1]) {
        closed.lower[i] <- FALSE
      }
    }

    for (i in 1:(ncol(pairs) - 1)) { # open upper inteval if below singlet
      if (pairs[2, i] == pairs[1, i + 1] && pairs[2, i] == pairs[2, i + 1]) {
        closed.upper[i] <- FALSE
      }
    }

    levs <- ifelse(pairs[1, ] == pairs[2, ],
      pairs[1, ],
      paste(ifelse(closed.lower, "[", "("),
        pairs[1, ],
        ",",
        pairs[2, ],
        ifelse(closed.upper, "]", ")"),
        sep = ""
      )
    )
    levels(retval) <- levs
  }
  else {
    retval <- cut(x, quant, include.lowest = TRUE, ...)
  }
  return(retval)
}

#' Most frequently occurring value
#'
#' @param x vector of values
#' @param na.rm logical.  Should \code{NA} values be removed before processing?
#' @param ties character. Which value(s) should be returned in the case of ties?
#' @param \dots optional additional parameters.
#'
#' @return vector of the same class as \code{x}
#'
#' @author Genei Ryodan and Gregory R. Warnes \email{greg@warnes.net}.
#'
#' @examples
#'
#' # Character vector
#' chr_vec <- c("a", "d", "d", "h", "h", NA, NA) # Multiple modes
#' stat_mode(x = chr_vec)
#' stat_mode(x = chr_vec, na.rm = FALSE)
#' stat_mode(x = chr_vec, na.rm = FALSE, ties = "first")
#' stat_mode(x = chr_vec, na.rm = FALSE, ties = "last")
#'
#' # - # Numeric vector
#' # See that it keeps the original vector type
#' num_vec <- c(2, 3, 3, 4, 4, NA, NA)
#' stat_mode(x = num_vec)
#' stat_mode(x = num_vec, na.rm = FALSE)
#' stat_mode(x = num_vec, na.rm = FALSE, ties = "first")
#' stat_mode(x = num_vec, na.rm = FALSE, ties = "last")
#'
#' # The default option is ties="all" but it is very easy for the user to control
#' # the ties without changing this parameter.
#' # Select always just one mode, being that the first mode
#' stat_mode(x = num_vec)[1]
#'
#' # Select the first and the second stat_mode
#' stat_mode(x = num_vec)[c(1, 2)]
#'
#' # Logical Vectors
#' stat_mode(x = c(TRUE, TRUE))
#' stat_mode(x = c(FALSE, FALSE, TRUE, TRUE))
#'
#' # - # Single element cases
#' stat_mode(x = c(NA_real_))
#' stat_mode(x = 2)
#' stat_mode(x = NA)
#' stat_mode(x = c("a"))
#'
#' # Not allowing multiple stat_mode, returning NA if that happens
#' stat_mode(x = c(1, 1, 2, 2), multiple_modes = FALSE) # multiple stat_mode
#' stat_mode(x = c(1, 1), multiple_modes = FALSE) # single mode
#'
#' # Empty vector cases
#' # The ties of any empty vector will be itself (an empty vector of the same type)
#' stat_mode(x = double())
#' stat_mode(x = complex())
#' stat_mode(x = vector("numeric"))
#' stat_mode(x = vector("character"))
#' @importFrom stats na.omit
#' @export
stat_mode <- function(x,
                      na.rm = TRUE,
                      ties = c("all", "first", "last", "missing"),
                      ...) {
  ties <- match.arg(ties)

  if (na.rm) {
    uv <- unique(na.omit(x))
  } else {
    uv <- unique(x)
  }

  tab <- tabulate(match(x, uv))

  all_modes <- uv[tab == max(tab)]

  if (length(all_modes) > 1) {
    if (ties == "first") {
      return(all_modes[1])
    } else if (ties == "last") {
      return(all_modes[length(all_modes)])
    } else if (ties == "all") {
      return(all_modes)
    } # ties=="missing"
    else {
      return(NA)
    }
  }
  else {
    return(all_modes)
  }
}

#' gtools: Various R Programming Tools
#'
#' Functions to assist in R programming, including:
#' \describe{
#' \item{developing, updating, and maintaining R and R packages}{'ask', 'checkRVersion', 'getDependencies', 'keywords', 'scat'}
#' \item{calculate the logit and inverse logit transformations}{'logit', 'inv.logit'}
#' \item{test if a value is missing, empty, contains only NA and NULL values, or is a 'try-error'}{'invalid'}
#' \item{manipulate R's .Last function}{'addLast'}
#' \item{define macros}{'defmacro', 'strmacro'}
#' \item{detect odd and even integers}{'odd', 'even'}
#' \item{convert strings containing non-ASCII characters (like single quotes) to plain ASCII}{'ASCIIfy'}
#' \item{perform a binary search}{'binsearch'}
#' \item{sort strings containing both numeric and character components}{'mixedsort', 'mixedorder'}
#' \item{create a factor variable from the quantiles of a continuous variable}{'quantcut'}
#' \item{enumerate permutations and combinations}{'combinations', 'permutation'}
#' \item{calculate and convert between fold-change and log-ratio}{'foldchange', 'logratio2foldchange', 'foldchange2logratio'}
#' \item{calculate probabilities and generate random numbers from Dirichlet distributions}{'rdirichlet', 'ddirichlet'}
#' \item{apply a function over adjacent subsets of a vector}{'running'}
#' \item{modify the TCP_NODELAY ('de-Nagle') flag for socket objects}{'tcpNoDelay'}
#' \item{efficient 'rbind' of data frames, even if the column names don't match}{'smartbind'}
#' \item{generate significance stars from p-values}{'stars.pval'}
#' \item{convert characters to/from ASCII codes}{asc, chr}
#' \item{convert character vector to ASCII representation}{'ASCIIfy'}
#' \item{apply title capitalization rules to a character vector}{'capwords'}
#' }
#'
#' @docType package
#' @name gtools
#' @useDynLib gtools, .registration=TRUE
#' @keywords internal
"_PACKAGE"

#' Binary Search
#'
#' Search within a specified range to locate an integer parameter which results
#' in the the specified monotonic function obtaining a given value.
#'
#' This function implements an extension to the standard binary search
#' algorithm for searching a sorted list.  The algorithm has been extended to
#' cope with cases where an exact match is not possible, to detect whether that
#' the function may be monotonic increasing or decreasing and act
#' appropriately, and to detect when the target value is outside the specified
#' range.
#'
#' The algorithm initializes two variable \code{lo} and \code{high} to the
#' extremes values of \code{range}.  It then generates a new value
#' \code{center} halfway between \code{lo} and \code{hi}.  If the value of
#' \code{fun} at \code{center} exceeds \code{target}, it becomes the new value
#' for \code{lo}, otherwise it becomes the new value for \code{hi}.  This
#' process is iterated until \code{lo} and \code{hi} are adjacent.  If the
#' function at one or the other equals the target, this value is returned,
#' otherwise \code{lo}, \code{hi}, and the function value at both are returned.
#'
#' Note that when the specified target value falls between integers, the
#' \emph{two} closest values are returned.  If the specified target falls
#' outside of the specified \code{range}, the closest endpoint of the range
#' will be returned, and an warning message will be generated.  If the maximum
#' number if iterations was reached, the endpoints of the current subset of the
#' range under consideration will be returned.
#'
#' @param fun Monotonic function over which the search will be performed.
#' @param range 2-element vector giving the range for the search.
#' @param \dots Additional parameters to the function \code{fun}.
#' @param target Target value for \code{fun}.  Defaults to 0.
#' @param lower Lower limit of search range. Defaults to \code{min(range)}.
#' @param upper Upper limit of search range. Defaults to \code{max(range)}.
#' @param maxiter Maximum number of search iterations. Defaults to 100.
#' @param showiter Boolean flag indicating whether the algorithm state should
#' be printed at each iteration. Defaults to FALSE.
#' @return A list containing: \item{call}{How the function was called.}
#' \item{numiter}{The number of iterations performed} \item{flag }{One of the
#' strings, "Found", "Between Elements", "Maximum number of iterations
#' reached", "Reached lower boundary", or "Reached upper boundary."}
#' \item{where}{One or two values indicating where the search terminated.}
#' \item{value}{Value of the function \code{fun} at the values of
#' \code{where}.}
#' @note This function often returns two values for \code{where} and
#' \code{value}.  Be sure to check the \code{flag} parameter to see what these
#' values mean.
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link{optim}}, \code{\link{optimize}}, \code{\link{uniroot}}
#' @keywords optimize programming
#' @examples
#'
#'
#' ### Toy examples
#'
#' # search for x=10
#' binsearch(function(x) x - 10, range = c(0, 20))
#'
#' # search for x=10.1
#' binsearch(function(x) x - 10.1, range = c(0, 20))
#'
#' ### Classical toy example
#'
#' # binary search for the index of 'M' among the sorted letters
#' fun <- function(X) {
#'   ifelse(LETTERS[X] > "M", 1,
#'     ifelse(LETTERS[X] < "M", -1, 0)
#'   )
#' }
#'
#' binsearch(fun, range = 1:26)
#' # returns $where=13
#' LETTERS[13]
#'
#' ### Substantive example, from genetics
#' \dontrun{
#' library(genetics)
#' # Determine the necessary sample size to detect all alleles with
#' # frequency 0.07 or greater with probability 0.95.
#' power.fun <- function(N) 1 - gregorius(N = N, freq = 0.07)$missprob
#'
#' binsearch(power.fun, range = c(0, 100), target = 0.95)
#'
#' # equivalent to
#' gregorius(freq = 0.07, missprob = 0.05)
#' }
#'
#' @export
binsearch <- function(fun, range, ..., target = 0,
                      lower = ceiling(min(range)), upper = floor(max(range)),
                      maxiter = 100, showiter = FALSE) {

  # initialize
  lo <- lower
  hi <- upper
  counter <- 0
  val.lo <- fun(lo, ...)
  val.hi <- fun(hi, ...)

  # check whether function is increasing or decreasing, & set sign
  # appropriately.
  if (val.lo > val.hi) {
    sign <- -1
  } else {
    sign <- 1
  }

  # check if value is outside specified range
  if (target * sign < val.lo * sign) {
    outside.range <- TRUE
  } else if (target * sign > val.hi * sign) {
    outside.range <- TRUE
  } else {
    outside.range <- FALSE
  }

  # iteratively move lo & high closer together until we run out of
  # iterations, or they are adjacent, or they are identical
  while (counter < maxiter && !outside.range) {
    counter <- counter + 1

    if (hi - lo <= 1 || lo < lower || hi > upper) break

    center <- round((hi - lo) / 2 + lo, 0)
    val <- fun(center, ...)

    if (showiter) {
      cat("--------------\n")
      cat("Iteration #", counter, "\n")
      cat("lo=", lo, "\n")
      cat("hi=", hi, "\n")
      cat("center=", center, "\n")
      cat("fun(lo)=", val.lo, "\n")
      cat("fun(hi)=", val.hi, "\n")
      cat("fun(center)=", val, "\n")
    }


    if (val == target) {
      val.lo <- val.hi <- val
      lo <- hi <- center
      break
    }
    else if (sign * val < sign * target) {
      lo <- center
      val.lo <- val
    }
    else # ( val > target )
    {
      hi <- center
      val.hi <- val
    }

    if (showiter) {
      cat("new lo=", lo, "\n")
      cat("new hi=", hi, "\n")
      cat("--------------\n")
    }
  }

  # Create return value
  retval <- list()
  retval$call <- match.call()
  retval$numiter <- counter

  if (outside.range) {
    if (target * sign < val.lo * sign) {
      warning("Reached lower boundary")
      retval$flag <- "Lower Boundary"
      retval$where <- lo
      retval$value <- val.lo
    }
    else # (target * sign >  val.hi * sign)
    {
      warning("Reached upper boundary")
      retval$flag <- "Upper Boundary"
      retval$where <- hi
      retval$value <- val.hi
    }
  }
  else if (counter >= maxiter) {
    warning("Maximum number of iterations reached")
    retval$flag <- "Maximum number of iterations reached"
    retval$where <- c(lo, hi)
    retval$value <- c(val.lo, val.hi)
  }
  else if (val.lo == target) {
    retval$flag <- "Found"
    retval$where <- lo
    retval$value <- val.lo
  }
  else if (val.hi == target) {
    retval$flag <- "Found"
    retval$where <- hi
    retval$value <- val.hi
  }
  else {
    retval$flag <- "Between Elements"
    retval$where <- c(lo, hi)
    retval$value <- c(val.lo, val.hi)
  }

  return(retval)
}

#' Dataset That Crashes Base:::Plot.Dendogram with 'Node Stack Overflow'
#'
#' Base:::Plot.Dendogram() will generate a 'Node Stack Overflow' when run on a
#' dendrogram appropriately constructed from this data set.
#'
#'
#' @name badDend
#' @docType data
#' @format The format is: num [1:2047, 1:12] 1 2 3 4 5 6 7 8 9 10 ...  -
#' attr(*, "dimnames")=List of 2 ..$ : NULL ..$ : chr [1:12] "X" "V1" "V2" "V3"
#' ...
#' @note See help page for \code{\link{unByteCode}} to see how to construct the
#' 'bad' dendrogram from this data and how to work around the issue.
#' @keywords datasets
#' @examples
#'
#' data(badDend)
NULL

#' Check if a newer version of R is available
#'
#' Check if a newer version of R is available
#'
#' This function accesses the R web site to discover the latest released
#' version of R.  It then compares this version to the running version.  If the
#' running version is the same as the latest version, it prints the message,
#' "The latest version of R is installed:" followed by the version number, and
#' returns NULL.  If the running version is older than the current version, it
#' displays the message, "A newer version of R is now available:" followed by
#' the corresponding version number, and returns the version number.
#'
#' If \code{quiet=TRUE}, no printing is performed.
#'
#' @param quiet Logical indicating whether printed output should be suppressed.
#' @return Either the version number of the latest version of R, if the running
#' version is less than the latest version, or NULL.
#' @note This function uses the internet to access the R project web site.
#' If internet access is unavailable or the R project web site is down, the function will fail.
#' @author Gregory R. Warnes
#' @seealso \code{\link[base]{R.Version}}
#' @keywords utilities
#' @examples
#'
#' try(
#'    ver <- checkRVersion()
#' )
#' print(ver)
#' @export
checkRVersion <- function(quiet = FALSE) {

  cran_page <- tryCatch(
      suppressWarnings(scan(
          file = "https://cran.r-project.org/src/base/R-4",
          what = "", quiet = TRUE)),
      error = function(cond) {
          stop("Your internet connection is not working (or CRAN is down)")
      }
  )

  matches <- grep("R-[0-9]\\.[0-9]+\\.[0-9]+", cran_page, value = TRUE)
  versionList <- gsub("^.*R-([0-9].[0-9]+.[0-9]+).*$", "\\1", matches)
  versionList <- numeric_version(versionList)
  if (max(versionList) > getRversion()) {
    if (!quiet) {
      cat("A newer version of R is now available: ")
      cat(as.character(max(versionList)))
      cat("\n")
    }
    invisible(max(versionList))
  }
  else {
    if (!quiet) {
      cat("The latest version of R is installed: ")
      cat(as.character(max(versionList)))
      cat("\n")
    }
    invisible(NULL)
  }
}

library("gtools")

host <- "www.r-project.org"

socket <- make.socket(host, 80)
print(socket)

setTCPNoDelay(socket, TRUE)

write.socket(socket, "GET http://www.r-project.org/index.html\r\n")
write.socket(socket, "HOST 127.0.0.1\r\n")
write.socket(socket, "\r\n")

# Read and display the response
while ((str <- read.socket(socket)) > "") {
  cat(str)
}

close.socket(socket)

#' Provide Name, Version, and Path of Loaded Package Namespaces
#'
#' Provide name, version, and path of loaded package namespaces
#'
#'
#' @param silent Logical indicating whether the results should be printed
#' @return Invisibly returns a data frame containing one row per loaded package
#' namespace, with columns: \item{Package}{Package name} \item{Version}{Version
#' string} \item{Path}{Path to package files} \item{SearchPath}{Either the
#' index of the package namespace in the current search path, or '-' if the
#' package namespace is not in the search path. '1' corresponds to the top of
#' the search path (the first namespace searched for values).  }
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link[base]{loadedNamespaces}},
#' \code{\link[utils]{packageVersion}}, \code{\link[base]{search}},
#' \code{\link[base]{find.package}}
#' @keywords package
#' @examples
#'
#' loadedPackages()
#' @importFrom utils packageVersion
#' @export
loadedPackages <- function(silent = FALSE) {
  packageNames <- loadedNamespaces()
  packageVersions <- sapply(packageNames, function(package) paste(packageVersion(package), sep = "."))
  packagePaths <- find.package(packageNames)
  inSearchPath <- match(packageNames, gsub("^package:", "", grep("^package:", search(), value = TRUE)))
  retval <- data.frame(Name = packageNames, Version = packageVersions, Path = packagePaths, SearchPath = inSearchPath)
  retval$SearchPath <- na.replace(retval$SearchPath, "-")
  retval <- retval[order(inSearchPath), ]
  if (!silent) print(retval)
  invisible(retval)
}

#' Order or Sort strings with embedded numbers so that the numbers are in the
#' correct order
#'
#' These functions sort or order character strings containing embedded numbers
#' so that the numbers are numerically sorted rather than sorted by character
#' value.  I.e. "Aspirin 50mg" will come before "Aspirin 100mg".  In addition,
#' case of character strings is ignored so that "a", will come before "B" and
#' "C".
#'
#' I often have character vectors (e.g. factor labels), such as compound and
#' dose, that contain both text and numeric data.  This function is useful for
#' sorting these character vectors into a logical order.
#'
#' It does so by splitting each character vector into a sequence of character
#' and numeric sections, and then sorting along these sections, with numbers
#' being sorted by numeric value (e.g. "50" comes before "100"), followed by
#' characters strings sorted by character value (e.g. "A" comes before "B")
#' \emph{ignoring case} (e.g. 'A' has the same sort order as 'a').
#'
#' By default, sort order is ascending, empty strings are sorted to the front,
#' and \code{NA} values to the end.  Setting \code{descending=TRUE} changes the
#' sort order to descending and reverses the meanings of \code{na.last} and
#' \code{blank.last}.
#'
#' Parsing looks for decimal numbers unless \code{numeric.type="roman"}, in
#' which parsing looks for roman numerals, with character case specified by
#' \code{roman.case}.
#'
#' @aliases mixedsort mixedorder
#' @param x Vector to be sorted.
#' @param decreasing logical.  Should the sort be increasing or decreasing?
#' Note that \code{descending=TRUE} reverses the meanings of \code{na.last} and
#' \code{blanks.last}.
#' @param na.last for controlling the treatment of \code{NA} values.  If
#' \code{TRUE}, missing values in the data are put last; if \code{FALSE}, they
#' are put first; if \code{NA}, they are removed.
#' @param blank.last for controlling the treatment of blank values.  If
#' \code{TRUE}, blank values in the data are put last; if \code{FALSE}, they
#' are put first; if \code{NA}, they are removed.
#' @param numeric.type either "decimal" (default) or "roman".  Are numeric
#' values represented as decimal numbers (\code{numeric.type="decimal"}) or as
#' Roman numerals (\code{numeric.type="roman"})?
#' @param roman.case one of "upper", "lower", or "both".  Are roman numerals
#' represented using only capital letters ('IX') or lower-case letters ('ix')
#' or both?
#' @param scientific logical. Should exponential notation be allowed for numeric values.
#' @return \code{mixedorder} returns a vector giving the sort order of the
#' input elements. \code{mixedsort} returns the sorted vector.
#' @author Gregory R. Warnes \email{greg@@warnes.net}
#' @seealso \code{\link[base]{sort}}, \code{\link[base]{order}}
#' @keywords univar manip
#' @examples
#'
#' ## compound & dose labels
#' Treatment <- c(
#'   "Control", "Aspirin 10mg/day", "Aspirin 50mg/day",
#'   "Aspirin 100mg/day", "Acetomycin 100mg/day",
#'   "Acetomycin 1000mg/day"
#' )
#'
#' ## ordinary sort puts the dosages in the wrong order
#' sort(Treatment)
#'
#' ## but mixedsort does the 'right' thing
#' mixedsort(Treatment)
#'
#' ## Here is a more complex example
#' x <- rev(c(
#'   "AA 0.50 ml", "AA 1.5 ml", "AA 500 ml", "AA 1500 ml",
#'   "EXP 1", "AA 1e3 ml", "A A A", "1 2 3 A", "NA", NA, "1e2",
#'   "", "-", "1A", "1 A", "100", "100A", "Inf"
#' ))
#'
#' mixedorder(x)
#'
#' mixedsort(x) # Notice that plain numbers, including 'Inf' show up
#' # before strings, NAs at the end, and blanks at the
#' # beginning .
#'
#'
#' mixedsort(x, na.last = TRUE) # default
#' mixedsort(x, na.last = FALSE) # push NAs to the front
#'
#'
#' mixedsort(x, blank.last = FALSE) # default
#' mixedsort(x, blank.last = TRUE) # push blanks to the end
#'
#' mixedsort(x, decreasing = FALSE) # default
#' mixedsort(x, decreasing = TRUE) # reverse sort order
#'
#' ## Roman numerals
#' chapters <- c(
#'   "V. Non Sequiturs", "II. More Nonsense",
#'   "I. Nonsense", "IV. Nonesensical Citations",
#'   "III. Utter Nonsense"
#' )
#' mixedsort(chapters, numeric.type = "roman")
#'
#' ## Lower-case Roman numerals
#' vals <- c(
#'   "xix", "xii", "mcv", "iii", "iv", "dcclxxii", "cdxcii",
#'   "dcxcviii", "dcvi", "cci"
#' )
#' (ordered <- mixedsort(vals, numeric.type = "roman", roman.case = "lower"))
#' roman2int(ordered)
#'
#' ## Control scientific notation for number matching:
#' vals <- c("3E1", "2E3", "4e0")
#'
#' mixedsort(vals) # With scientfic notation
#' mixedsort(vals, scientific = FALSE) # Without scientfic notation
#' @export
mixedsort <- function(x,
                      decreasing = FALSE,
                      na.last = TRUE,
                      blank.last = FALSE,
                      numeric.type = c("decimal", "roman"),
                      roman.case = c("upper", "lower", "both"),
                      scientific = TRUE) {
  ord <- mixedorder(x,
    decreasing = decreasing,
    na.last = na.last,
    blank.last = blank.last,
    numeric.type = numeric.type,
    roman.case = roman.case,
    scientific = scientific
  )
  x[ord]
}

#' @rdname mixedsort
#' @export
mixedorder <- function(x,
                       decreasing = FALSE,
                       na.last = TRUE,
                       blank.last = FALSE,
                       numeric.type = c("decimal", "roman"),
                       roman.case = c("upper", "lower", "both"),
                       scientific = TRUE) {
  # - Split each each character string into an vector of strings and
  #   numbers
  # - Separately rank numbers and strings
  # - Combine orders so that strings follow numbers

  numeric.type <- match.arg(numeric.type)
  roman.case <- match.arg(roman.case)

  if (length(x) < 1) {
    return(NULL)
  } else if (length(x) == 1) {
    return(1)
  }

  if (!is.character(x)) {
    return(order(x, decreasing = decreasing, na.last = na.last))
  }

  delim <- "\\$\\@\\$"

  if (numeric.type == "decimal") {
    if (scientific) {
      regex <- "((?:(?i)(?:[-+]?)(?:(?=[.]?[0123456789])(?:[0123456789]*)(?:(?:[.])(?:[0123456789]{0,}))?)(?:(?:[eE])(?:(?:[-+]?)(?:[0123456789]+))|)))"
    } # uses PERL syntax
    else {
      regex <- "((?:(?i)(?:[-+]?)(?:(?=[.]?[0123456789])(?:[0123456789]*)(?:(?:[.])(?:[0123456789]{0,}))?)))"
    } # uses PERL syntax

    numeric <- function(x) as.numeric(x)
  }
  else if (numeric.type == "roman") {
    regex <- switch(roman.case,
      "both"  = "([IVXCLDMivxcldm]+)",
      "upper" = "([IVXCLDM]+)",
      "lower" = "([ivxcldm]+)"
    )
    numeric <- function(x) roman2int(x)
  }
  else {
    stop("Unknown value for numeric.type: ", numeric.type)
  }

  nonnumeric <- function(x) {
    ifelse(is.na(numeric(x)), toupper(x), NA)
  }

  x <- as.character(x)

  which.nas <- which(is.na(x))
  which.blanks <- which(x == "")

  ####
  # - Convert each character string into an vector containing single
  #   character and  numeric values.
  ####

  # find and mark numbers in the form of +1.23e+45.67
  delimited <- gsub(regex,
    paste(delim, "\\1", delim, sep = ""),
    x,
    perl = TRUE
  )

  # separate out numbers
  step1 <- strsplit(delimited, delim)

  # remove empty elements
  step1 <- lapply(step1, function(x) x[x > ""])

  # create numeric version of data
  suppressWarnings(step1.numeric <- lapply(step1, numeric))

  # create non-numeric version of data
  suppressWarnings(step1.character <- lapply(step1, nonnumeric))

  # now transpose so that 1st vector contains 1st element from each
  # original string
  maxelem <- max(sapply(step1, length))

  step1.numeric.t <- lapply(
    1:maxelem,
    function(i) {
      sapply(
        step1.numeric,
        function(x) x[i]
      )
    }
  )

  step1.character.t <- lapply(
    1:maxelem,
    function(i) {
      sapply(
        step1.character,
        function(x) x[i]
      )
    }
  )

  # now order them
  rank.numeric <- sapply(step1.numeric.t, rank)
  rank.character <- sapply(
    step1.character.t,
    function(x) as.numeric(factor(x))
  )

  # and merge
  rank.numeric[!is.na(rank.character)] <- 0 # mask off string values

  rank.character <- t(
    t(rank.character) +
      apply(matrix(rank.numeric), 2, max, na.rm = TRUE)
  )

  rank.overall <- ifelse(is.na(rank.character), rank.numeric, rank.character)

  order.frame <- as.data.frame(rank.overall)
  if (length(which.nas) > 0) {
    if (is.na(na.last)) {
      order.frame[which.nas, ] <- NA
    } else if (na.last) {
      order.frame[which.nas, ] <- Inf
    } else {
      order.frame[which.nas, ] <- -Inf
    }
  }

  if (length(which.blanks) > 0) {
    if (is.na(blank.last)) {
      order.frame[which.blanks, ] <- NA
    } else if (blank.last) {
      order.frame[which.blanks, ] <- 1e99
    } else {
      order.frame[which.blanks, ] <- -1e99
    }
  }

  order.frame <- as.list(order.frame)
  order.frame$decreasing <- decreasing
  order.frame$na.last <- NA

  retval <- do.call("order", order.frame)

  return(retval)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/baseOf.R
\name{baseOf}
\alias{baseOf}
\title{Transform an integer to an array of base-n digits}
\usage{
baseOf(v, base = 10, len = 1)
}
\arguments{
\item{v}{A single integer value to be transformed.}

\item{base}{The base to which to transform to.}

\item{len}{The minimal length of the returned array.}
}
\description{
Transform an integer to an array of base-n digits
}
\details{
This function converts the elements of an integer vector as an array of its
digits.  The base of the numbering scheme may be changed away from 10, which
defines our decimal system, to any other integer value. For base=2, the
number is returned in the binary system. The least significant digit has the
highest index in the array, i.e. it appears on the right.  The highest
exponent is at position 1, i.e. left.

To write decimal values in another base is very common in computer science.
In particular at the basis 2 the then possible values 0 and 1 are often
interpreted as logical false or true. And at the very interface to
electrical engineering, it is indicated as an absence or presence of
voltage. When several bit values are transported synchronously, then it is
common to give every lane of such a data bus a unique 2^x value and
interpret it as a number in the binary system. To distinguish 256 characters
one once needed 8 bit ("byte"). It is the common unit in which larger
non-printable data is presented.  Because of the many non-printable
characters and the difficulty for most humans to memorize an even longer
alphabet, it is presented as two half bytes ("nibble") of 4 bit in a
hexadecimal presentation. Example code is shown below.

For statisticians, it is more likely to use bit representations for hashing.
A bit set to 1 (TRUE) at e.g. position 2, 9 or 17 is interpreted as the
presence of a particular feature combination of a sample.  With baseOf, you
can refer to the bit combination as a number, which is more easily and more
efficiently dealt with than with an array of binary values. The example code
presents a counter of combinations of features which may be interpreted as a
Venn diagram.
}
\examples{

# decimal representation
baseOf(123)

# binary representation
baseOf(123, base = 2)

# octal representation
baseOf(123, base = 8)

# hexadecimal representation
baseOf(123, base = 16)

# hexadecimal with more typical letter-notation
c(0:9, LETTERS)[baseOf(123, 16)]

# hexadecimal again, now showing a single string
paste(c(0:9, LETTERS)[baseOf(123, 16)], collapse = "")

# decimal representation but filling leading zeroes
baseOf(123, len = 5)

# and converting that back
sum(2^(4:0) * baseOf(123, len = 5))

# hashing and a tabular venn diagram derived from it
m <- matrix(sample(c(FALSE, TRUE), replace = TRUE, size = 300), ncol = 4)
colnames(m) <- c("strong", "colorful", "nice", "humorous")
names(dimnames(m)) <- c("samples", "features")
head(m)

m.val <- apply(m, 1, function(X) {
  return(sum(2^((ncol(m) - 1):0) * X))
})
m.val.rle <- rle(sort(m.val))
m.counts <- cbind(
  baseOf(m.val.rle$value, base = 2, len = ncol(m)),
  m.val.rle$lengths
)
colnames(m.counts) <- c(colnames(m), "num")
rownames(m.counts) <- apply(m.counts[, 1:ncol(m)], 1, paste, collapse = "")
m.counts[1 == m.counts[, "nice"] & 1 == m.counts[, "humorous"], , drop = FALSE]
m.counts[, "num", drop = TRUE]
}
\author{
Steffen Moeller \email{moeller@debian.org}
}
\keyword{base}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asc.R
\name{asc}
\alias{asc}
\alias{chr}
\title{Convert between characters and ASCII codes}
\usage{
asc(char, simplify = TRUE)

chr(ascii)
}
\arguments{
\item{char}{vector of character strings}

\item{simplify}{logical indicating whether to attempt to convert the result
into a vector or matrix object. See \code{\link[base]{sapply}} for details.}

\item{ascii}{vector or list of vectors containing integer ASCII codes}
}
\value{
\code{asc} returns the integer ASCII values for each character in
the elements of \code{char}.  If \code{simplify=FALSE} the result will be a
list containing one vector per element of \code{char}.  If
\code{simplify=TRUE}, the code will attempt to convert the result into a
vector or matrix.

\code{asc} returns the characters corresponding to the provided ASCII
values.
}
\description{
Convert between characters and ASCII codes
}
\section{Functions}{
\itemize{
\item \code{asc()}: return the characters corresponding to the specified ASCII codes

\item \code{chr()}: return the ASCII codes for the specified characters.

}}
\examples{

## ascii codes for lowercase letters
asc(letters)

## uppercase letters from ascii codes
chr(65:90)

## works on muti-character strings
(tmp <- asc("hello!"))
chr(tmp)

## Use 'simplify=FALSE' to return the result as a list
(tmp <- asc("hello!", simplify = FALSE))
chr(tmp)

## When simplify=FALSE the results can be...
asc(c("a", "e", "i", "o", "u", "y")) # a vector
asc(c("ae", "io", "uy")) # or a matrix

## When simplify=TRUE the results are always a list...
asc(c("a", "e", "i", "o", "u", "y"), simplify = FALSE)
asc(c("ae", "io", "uy"), simplify = FALSE)
}
\seealso{
\code{\link[base]{strtoi}}, \code{\link[base]{charToRaw}},
\code{\link[base]{rawToChar}}, \code{\link[base]{as.raw}}
}
\author{
Adapted by Gregory R. Warnes \email{greg@warnes.net} from code
posted by Mark Davis on the 'Data Debrief' blog on 2011-03-09 at
\url{https://datadebrief.blogspot.com/2011/03/ascii-code-table-in-r.html}.
}
\keyword{character}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/split_path.R
\name{split_path}
\alias{split_path}
\title{Split a File Path into Components}
\usage{
split_path(x, depth_first = TRUE)
}
\arguments{
\item{x}{character scalar.  Path to be processed.}

\item{depth_first}{logical.  Should path be returned depth first?  Defaults
to \code{TRUE}.}
}
\value{
Character vector of path components, depth first.
}
\description{
This function converts a character scalar containing a
 \emph{valid} file path into a character vector of path components
 (e.g. directories).
}
\examples{
    longPath = "/users/me/src/this/is/a/long/path"
    split_path(longPath)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/script_file.R
\name{script_file}
\alias{script_file}
\alias{script_path}
\title{Determine the directory or full path to the currently executing script}
\usage{
script_file(fail = c("stop", "warning", "quiet"))

script_path(fail = c("stop", "warning", "quiet"))
}
\arguments{
\item{fail}{character, one of "stop", "warning", "quiet". specifying what
 should be done when the script path cannot be determined:
 "stop" causes an error to be generated,
 "warn" generates a warning message and returns NA,
 "quiet" silently returns NA.

These function should work with \code{Rscript}, \code{source()},
\code{Rmarkdown} \code{RStudio}'s "Run selection", and the \code{RStudio}
Console.}
}
\value{
A character scalar containing the full path to the currently
  executing script file (\code{script_file}) or its directory
  (\code{script_path}).  If unable to determine the script path, it generates
  a warning and returns \code{""} (empty string).
}
\description{
Determine the directory or full path to the currently executing script
}
\section{Functions}{
\itemize{
\item \code{script_file()}: Determine the full path of the currently executing
script

\item \code{script_path()}: Determine the directory of the currently executing script

}}
\examples{

getwd()
commandArgs(trailingOnly = FALSE)

script_file("warning")
script_path("warning")
}
\author{
Greg Warnes \email{greg@warnes.net} based on on a Stack Overflow post by
  jerry-t (\url{https://stackoverflow.com/users/2292993/jerry-t}) at
  \url{https://stackoverflow.com/a/36777602/2744062}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stat_mode.R
\name{stat_mode}
\alias{stat_mode}
\title{Most frequently occurring value}
\usage{
stat_mode(x, na.rm = TRUE, ties = c("all", "first", "last", "missing"), ...)
}
\arguments{
\item{x}{vector of values}

\item{na.rm}{logical.  Should \code{NA} values be removed before processing?}

\item{ties}{character. Which value(s) should be returned in the case of ties?}

\item{\dots}{optional additional parameters.}
}
\value{
vector of the same class as \code{x}
}
\description{
Most frequently occurring value
}
\examples{

# Character vector
chr_vec <- c("a", "d", "d", "h", "h", NA, NA) # Multiple modes
stat_mode(x = chr_vec)
stat_mode(x = chr_vec, na.rm = FALSE)
stat_mode(x = chr_vec, na.rm = FALSE, ties = "first")
stat_mode(x = chr_vec, na.rm = FALSE, ties = "last")

# - # Numeric vector
# See that it keeps the original vector type
num_vec <- c(2, 3, 3, 4, 4, NA, NA)
stat_mode(x = num_vec)
stat_mode(x = num_vec, na.rm = FALSE)
stat_mode(x = num_vec, na.rm = FALSE, ties = "first")
stat_mode(x = num_vec, na.rm = FALSE, ties = "last")

# The default option is ties="all" but it is very easy for the user to control
# the ties without changing this parameter.
# Select always just one mode, being that the first mode
stat_mode(x = num_vec)[1]

# Select the first and the second stat_mode
stat_mode(x = num_vec)[c(1, 2)]

# Logical Vectors
stat_mode(x = c(TRUE, TRUE))
stat_mode(x = c(FALSE, FALSE, TRUE, TRUE))

# - # Single element cases
stat_mode(x = c(NA_real_))
stat_mode(x = 2)
stat_mode(x = NA)
stat_mode(x = c("a"))

# Not allowing multiple stat_mode, returning NA if that happens
stat_mode(x = c(1, 1, 2, 2), multiple_modes = FALSE) # multiple stat_mode
stat_mode(x = c(1, 1), multiple_modes = FALSE) # single mode

# Empty vector cases
# The ties of any empty vector will be itself (an empty vector of the same type)
stat_mode(x = double())
stat_mode(x = complex())
stat_mode(x = vector("numeric"))
stat_mode(x = vector("character"))
}
\author{
Genei Ryodan and Gregory R. Warnes \email{greg@warnes.net}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binsearch.R
\name{binsearch}
\alias{binsearch}
\title{Binary Search}
\usage{
binsearch(
  fun,
  range,
  ...,
  target = 0,
  lower = ceiling(min(range)),
  upper = floor(max(range)),
  maxiter = 100,
  showiter = FALSE
)
}
\arguments{
\item{fun}{Monotonic function over which the search will be performed.}

\item{range}{2-element vector giving the range for the search.}

\item{\dots}{Additional parameters to the function \code{fun}.}

\item{target}{Target value for \code{fun}.  Defaults to 0.}

\item{lower}{Lower limit of search range. Defaults to \code{min(range)}.}

\item{upper}{Upper limit of search range. Defaults to \code{max(range)}.}

\item{maxiter}{Maximum number of search iterations. Defaults to 100.}

\item{showiter}{Boolean flag indicating whether the algorithm state should
be printed at each iteration. Defaults to FALSE.}
}
\value{
A list containing: \item{call}{How the function was called.}
\item{numiter}{The number of iterations performed} \item{flag }{One of the
strings, "Found", "Between Elements", "Maximum number of iterations
reached", "Reached lower boundary", or "Reached upper boundary."}
\item{where}{One or two values indicating where the search terminated.}
\item{value}{Value of the function \code{fun} at the values of
\code{where}.}
}
\description{
Search within a specified range to locate an integer parameter which results
in the the specified monotonic function obtaining a given value.
}
\details{
This function implements an extension to the standard binary search
algorithm for searching a sorted list.  The algorithm has been extended to
cope with cases where an exact match is not possible, to detect whether that
the function may be monotonic increasing or decreasing and act
appropriately, and to detect when the target value is outside the specified
range.

The algorithm initializes two variable \code{lo} and \code{high} to the
extremes values of \code{range}.  It then generates a new value
\code{center} halfway between \code{lo} and \code{hi}.  If the value of
\code{fun} at \code{center} exceeds \code{target}, it becomes the new value
for \code{lo}, otherwise it becomes the new value for \code{hi}.  This
process is iterated until \code{lo} and \code{hi} are adjacent.  If the
function at one or the other equals the target, this value is returned,
otherwise \code{lo}, \code{hi}, and the function value at both are returned.

Note that when the specified target value falls between integers, the
\emph{two} closest values are returned.  If the specified target falls
outside of the specified \code{range}, the closest endpoint of the range
will be returned, and an warning message will be generated.  If the maximum
number if iterations was reached, the endpoints of the current subset of the
range under consideration will be returned.
}
\note{
This function often returns two values for \code{where} and
\code{value}.  Be sure to check the \code{flag} parameter to see what these
values mean.
}
\examples{


### Toy examples

# search for x=10
binsearch(function(x) x - 10, range = c(0, 20))

# search for x=10.1
binsearch(function(x) x - 10.1, range = c(0, 20))

### Classical toy example

# binary search for the index of 'M' among the sorted letters
fun <- function(X) {
  ifelse(LETTERS[X] > "M", 1,
    ifelse(LETTERS[X] < "M", -1, 0)
  )
}

binsearch(fun, range = 1:26)
# returns $where=13
LETTERS[13]

### Substantive example, from genetics
\dontrun{
library(genetics)
# Determine the necessary sample size to detect all alleles with
# frequency 0.07 or greater with probability 0.95.
power.fun <- function(N) 1 - gregorius(N = N, freq = 0.07)$missprob

binsearch(power.fun, range = c(0, 100), target = 0.95)

# equivalent to
gregorius(freq = 0.07, missprob = 0.05)
}

}
\seealso{
\code{\link{optim}}, \code{\link{optimize}}, \code{\link{uniroot}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{optimize}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkRVersion.R
\name{checkRVersion}
\alias{checkRVersion}
\title{Check if a newer version of R is available}
\usage{
checkRVersion(quiet = FALSE)
}
\arguments{
\item{quiet}{Logical indicating whether printed output should be suppressed.}
}
\value{
Either the version number of the latest version of R, if the running
version is less than the latest version, or NULL.
}
\description{
Check if a newer version of R is available
}
\details{
This function accesses the R web site to discover the latest released
version of R.  It then compares this version to the running version.  If the
running version is the same as the latest version, it prints the message,
"The latest version of R is installed:" followed by the version number, and
returns NULL.  If the running version is older than the current version, it
displays the message, "A newer version of R is now available:" followed by
the corresponding version number, and returns the version number.

If \code{quiet=TRUE}, no printing is performed.
}
\note{
This function uses the internet to access the R project web site.
If internet access is unavailable or the R project web site is down, the function will fail.
}
\examples{

try(
   ver <- checkRVersion()
)
print(ver)
}
\seealso{
\code{\link[base]{R.Version}}
}
\author{
Gregory R. Warnes
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/foldchange.R
\name{foldchange}
\alias{foldchange}
\alias{logratio2foldchange}
\alias{foldchange2logratio}
\title{Compute fold-change or convert between log-ratio and fold-change.}
\usage{
foldchange(num, denom)

logratio2foldchange(logratio, base = 2)

foldchange2logratio(foldchange, base = 2)
}
\arguments{
\item{num, denom}{vector/matrix of numeric values}

\item{logratio}{vector/matrix of log-ratio values}

\item{base}{Exponential base for the log-ratio.}

\item{foldchange}{vector/matrix of fold-change values}
}
\value{
A vector or matrix of the same dimensions as the input containing
the converted values.
}
\description{
\code{foldchange} computes the fold change for two sets of values.
\code{logratio2foldchange} converts values from log-ratios to fold changes.
\code{foldchange2logratio} does the reverse.
}
\details{
Fold changes are commonly used in the biological sciences as a mechanism for
comparing the relative size of two measurements.  They are computed as:
\eqn{\frac{num}{denom}}{num/denom} if \eqn{num>denom}, and as
\eqn{\frac{-denom}{num}}{-denom/num} otherwise.

Fold-changes have the advantage of ease of interpretation and symmetry about
\eqn{num=denom}, but suffer from a discontinuity between -1 and 1, which can
cause significant problems when performing data analysis.  Consequently
statisticians prefer to use log-ratios.
}
\section{Functions}{
\itemize{
\item \code{foldchange()}: Compute fold-change.

\item \code{logratio2foldchange()}: Compute foldchange from log-ratio values.

\item \code{foldchange2logratio()}: Compute log-ratio from fold-change values.

}}
\examples{


a <- 1:21
b <- 21:1

f <- foldchange(a, b)

cbind(a, b, f)
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{math}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dirichlet.R
\name{dirichlet}
\alias{dirichlet}
\alias{rdirichlet}
\alias{ddirichlet}
\title{Functions for the Dirichlet Distribution}
\usage{
ddirichlet(x, alpha)

rdirichlet(n, alpha)
}
\arguments{
\item{x}{A vector containing a single random deviate or matrix containing one
random deviate per row.}

\item{alpha}{Vector or (for \code{ddirichlet}) matrix containing shape
parameters.}

\item{n}{Number of random vectors to generate.}
}
\value{
\code{ddirichlet} returns a vector containing the Dirichlet density
for the corresponding rows of \code{x}.

\code{rdirichlet} returns a matrix with \code{n} rows, each containing a
single Dirichlet random deviate.
}
\description{
Functions to compute the density of or generate random deviates from the
Dirichlet distribution
}
\details{
The Dirichlet distribution is the multidimensional generalization of the
beta distribution.  It is the canonical Bayesian distribution for the
parameter estimates of a multinomial distribution.
}
\section{Functions}{
\itemize{
\item \code{ddirichlet()}: Dirichlet distribution function.

\item \code{rdirichlet()}: Generate dirichlet random deviates.

}}
\examples{


x <- rdirichlet(20, c(1, 1, 1))

ddirichlet(x, c(1, 1, 1))
}
\seealso{
\code{\link{dbeta}}, \code{\link{rbeta}}
}
\author{
Code original posted by Ben Bolker to R-News on Fri Dec 15 2000. See
\url{https://stat.ethz.ch/pipermail/r-help/2000-December/009561.html}.  Ben
attributed the code to Ian Wilson \email{i.wilson@maths.abdn.ac.uk}.
Subsequent modifications by Gregory R. Warnes \email{greg@warnes.net}.
}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loadedPackages.R
\name{loadedPackages}
\alias{loadedPackages}
\title{Provide Name, Version, and Path of Loaded Package Namespaces}
\usage{
loadedPackages(silent = FALSE)
}
\arguments{
\item{silent}{Logical indicating whether the results should be printed}
}
\value{
Invisibly returns a data frame containing one row per loaded package
namespace, with columns: \item{Package}{Package name} \item{Version}{Version
string} \item{Path}{Path to package files} \item{SearchPath}{Either the
index of the package namespace in the current search path, or '-' if the
package namespace is not in the search path. '1' corresponds to the top of
the search path (the first namespace searched for values).  }
}
\description{
Provide name, version, and path of loaded package namespaces
}
\examples{

loadedPackages()
}
\seealso{
\code{\link[base]{loadedNamespaces}},
\code{\link[utils]{packageVersion}}, \code{\link[base]{search}},
\code{\link[base]{find.package}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setTCPNoDelay.R
\name{setTCPNoDelay}
\alias{setTCPNoDelay}
\title{Modify the TCP_NODELAY (`de-Nagle') flag for socket objects}
\usage{
setTCPNoDelay(socket, value = TRUE)
}
\arguments{
\item{socket}{A socket connection object}

\item{value}{Logical indicating whether to set (\code{TRUE}) or unset
(\code{FALSE}) the flag}
}
\value{
The character string "SUCCESS" will be returned invisible if the
operation was successful.  On failure, an error will be generated.
}
\description{
Modify the TCP_NODELAY (`de-Nagle') flag for socket objects
}
\details{
By default, TCP connections wait a small fixed interval before actually
sending data, in order to permit small packets to be combined.  This
algorithm is named after its inventor, John Nagle, and is often referred to
as 'Nagling'.

While this reduces network resource utilization in these situations, it
imposes a delay on all outgoing message data, which can cause problems in
client/server situations.

This function allows this feature to be disabled (de-Nagling,
\code{value=TRUE}) or enabled (Nagling, \code{value=FALSE}) for the
specified socket.
}
\examples{
\dontrun{
host <- "www.r-project.org"
socket <- make.socket(host, 80)
print(socket)
setTCPNoDelay(socket, TRUE)

write.socket(socket, "GET /\n\n")
write.socket(socket, "A")
write.socket(socket, "B\n")
while ((str <- read.socket(socket)) > "") {
  cat(str)
}
close.socket(socket)
}

}
\references{
"Nagle's algorithm" \url{https://en.wikipedia.org/wiki/Nagle's_algorithm},

  Nagle, John. "Congestion Control in IP/TCP Internetworks", IETF Request for
  Comments 896, January 1984. \url{https://www.ietf.org/rfc/rfc0896.txt?number=896}
}
\seealso{
\code{\link{make.socket}}, \code{\link{socketConnection}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{misc}
\keyword{programming}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gtools-deprecated.R
\name{gtools-deprecated}
\alias{gtools-deprecated}
\title{Deprecated Functions in the gtools package}
\description{
These functions are provided for compatibility with older versions of
gtools, and may be defunct as soon as the next release.
}
\details{
gtools currently contains no deprecated functions.

% The original help page for these functions is often % available at
\code{help("oldName-deprecated")} (note the quotes). % % \itemize{ % \item{}
% }
}
\seealso{
\code{\link{Deprecated}}
}
\keyword{misc}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combinations.R
\name{combinations}
\alias{combinations}
\alias{permutations}
\title{Enumerate the Combinations or Permutations of the Elements of a Vector}
\usage{
combinations(n, r, v = 1:n, set = TRUE, repeats.allowed = FALSE)

permutations(n, r, v = 1:n, set = TRUE, repeats.allowed = FALSE)
}
\arguments{
\item{n}{Size of the source vector}

\item{r}{Size of the target vectors}

\item{v}{Source vector. Defaults to \code{1:n}}

\item{set}{Logical flag indicating whether duplicates should be removed from
the source vector \code{v}. Defaults to \code{TRUE}.}

\item{repeats.allowed}{Logical flag indicating whether the constructed
vectors may include duplicated values.  Defaults to \code{FALSE}.}
}
\value{
Returns a matrix where each row contains a vector of length
\code{r}.
}
\description{
\code{combinations} enumerates the possible combinations of a specified size
from the elements of a vector.  \code{permutations} enumerates the possible
permutations.
}
\details{
Caution: The number of combinations and permutations increases rapidly with
\code{n} and \code{r}!.

To use values of \code{n} above about 45, you will need to increase R's
recursion limit.  See the \code{expression} argument to the \code{options}
command for details on how to do this.



Taken from an email by Brian D Ripley <ripley@stats.ox.ac.uk> to r-help
dated Tue, 14 Dec 1999 11:14:04 +0000 (GMT) in response to
Alex Ahgarin \email{datamanagement@email.com}.  Original version was
named "subsets" and was Written by Bill Venables.
}
\examples{

combinations(3,2,letters[1:3])
combinations(3,2,letters[1:3],repeats=TRUE)

permutations(3,2,letters[1:3])
permutations(3,2,letters[1:3],repeats=TRUE)

\dontrun{
# To use large 'n', you need to change the default recusion limit
options(expressions=1e5)
cmat <- combinations(300,2)
dim(cmat) # 44850 by 2 
}

}
\references{
Venables, Bill.  "Programmers Note", R-News, Vol 1/1, Jan. 2001.
\url{https://cran.r-project.org/doc/Rnews/}
}
\seealso{
\code{\link[base]{choose}}, \code{\link[base]{options}}
}
\author{
Original versions by Bill Venables
\email{Bill.Venables@cmis.csiro.au}.  Extended to handle
\code{repeats.allowed} by Gregory R. Warnes \email{greg@warnes.net}.
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/capwords.R
\name{capwords}
\alias{capwords}
\title{Capitalize Words for Titles}
\usage{
capwords(
  s,
  strict = FALSE,
  AP = TRUE,
  onlyfirst = FALSE,
  preserveMixed = FALSE,
  sep = " "
)
}
\arguments{
\item{s}{character string to be processed}

\item{strict}{Logical, remove all additional capitalization.}

\item{AP}{Logical, apply the Associated Press (AP) rules for prepositions
and conjunctions that should not be capitalized in titles.}

\item{onlyfirst}{Logical, only capitalize the first word.}

\item{preserveMixed}{Logical, preserve the capitalization mixed-case words
containing an upper-case letter after a lower-case letter.}

\item{sep}{Character string, word separator}
}
\value{
A character scalar containing the capitalized words.
}
\description{
This function capitalizes words for use in titles
}
\details{
This function separates the provided character string into separate words
using \code{sep} as the word separator.  If \code{firstonly==TRUE}, it then
capitalizes the first letter the first word, otherwise (the default), it
capitalizes the first letter of every word.  If \code{AP==TRUE}, it then
un-capitalizes words in the Associated Press's (AP) list of prepositions and
conjunctions should not be capitalized in titles.  Next, it capitalizes the
first word. It then re-joins the words using the specified separator.

If \code{preserveMixed==TRUE}, words with an upper-case letter appearing
after a lower-case letter will not be changed (e.g. "iDevice").
}
\examples{

capwords("a function to capitalize words in a title")
capwords("a function to capitalize words in a title", AP = FALSE)

capwords("testing the iProduct for defects")
capwords("testing the iProduct for defects", strict = TRUE)
capwords("testing the iProduct for defects", onlyfirst = TRUE)
capwords("testing the iProduct for defects", preserveMixed = TRUE)

capwords("title_using_underscores_as_separators", sep = "_")
}
\references{
Fogarty, Mignon. Capitalizing Titles: "Which words should you
capitalize? Grammar Girl's Quick and Dirty Tips for Better Writing. 9 Jun.
2011. Quick and Dirty Tips Website." Accessed 22 April 2016
\url{https://www.quickanddirtytips.com/articles/capitalizing-titles/}
}
\seealso{
\code{\link[base]{chartr}}, \code{\link[taxize]{taxize_capwords}},
\code{\link[SGP]{capwords}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net} based on code from the
\code{\link[base]{chartr}} manual page, and Scott Chamberlain's
\code{\link[taxize]{taxize_capwords}} in the taxize package.
}
\keyword{character}
\keyword{utilites}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/invalid.R
\name{invalid}
\alias{invalid}
\title{Test if a value is missing, empty, contains only NA or NULL values, or is a try-error.}
\usage{
invalid(x)
}
\arguments{
\item{x}{value to be tested}
}
\value{
Logical value.
}
\description{
Test if a value is missing, empty, contains only NA or NULL values, or is a try-error.
}
\examples{


invalid(NA)
invalid()
invalid(c(NA, NA, NULL, NA))

invalid(list(a = 1, b = NULL))

x <- try(log("A"))
invalid(x)

# example use in a function
myplot <- function(x, y) {
  if (invalid(y)) {
    y <- x
    x <- 1:length(y)
  }
  plot(x, y)
}
myplot(1:10)
myplot(1:10, NA)
}
\seealso{
\code{\link[base]{missing}}, \code{\link[base]{is.na}},
\code{\link[base]{is.null}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/badDend.R
\docType{data}
\name{badDend}
\alias{badDend}
\title{Dataset That Crashes Base:::Plot.Dendogram with 'Node Stack Overflow'}
\format{
The format is: num [1:2047, 1:12] 1 2 3 4 5 6 7 8 9 10 ...  -
attr(*, "dimnames")=List of 2 ..$ : NULL ..$ : chr [1:12] "X" "V1" "V2" "V3"
...
}
\description{
Base:::Plot.Dendogram() will generate a 'Node Stack Overflow' when run on a
dendrogram appropriately constructed from this data set.
}
\note{
See help page for \code{\link{unByteCode}} to see how to construct the
'bad' dendrogram from this data and how to work around the issue.
}
\examples{

data(badDend)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ask.R
\name{ask}
\alias{ask}
\title{Display a prompt and collect the user's response}
\usage{
ask(msg = "Press <RETURN> to continue: ", con = stdin())
}
\arguments{
\item{msg}{Character vector providing the message to be displayed}

\item{con}{Character connection to query, defaults to \code{stdin()}.}
}
\value{
A character scalar containing the input provided by the user.
}
\description{
Display a prompt and collect the user's response
}
\details{
The prompt message will be displayed, and then \code{readLines} is used to
collect a single input value (possibly empty), which is then returned.

In most situations using the default \code{con=stdin()} should work
properly.  Under RStudio, it is necessary to specify
\code{con=file("stdin")} for proper operation.
}
\examples{


# use default prompt
ask()

silly <- function() {
  age <- ask("How old aroe you? ")
  age <- as.numeric(age)
  cat("In 10 years you will be", age + 10, "years old!\n")
}
}
\seealso{
\code{\link{readLines}}, \code{\link{scan}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{IO}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scat.R
\name{scat}
\alias{scat}
\title{Display debugging text}
\usage{
scat(...)
}
\arguments{
\item{\dots}{Arguments passed to \code{cat}}
}
\value{
NULL (invisibly)
}
\description{
If \code{getOption('DEBUG')==TRUE}, write text to STDOUT and flush so that
the text is immediately displayed. Otherwise, do nothing.
}
\examples{

options(DEBUG = NULL) # makee sure DEBUG isn't set
scat("Not displayed")

options(DEBUG = TRUE)
scat("This will be displayed immediately (even in R BATCH output \n")
scat("files), provided options()$DEBUG is TRUE.")
}
\seealso{
\code{\link[base]{cat}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{print}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logit.R
\name{logit}
\alias{logit}
\alias{inv.logit}
\title{Generalized logit and inverse logit function}
\usage{
logit(x, min = 0, max = 1)

inv.logit(x, min = 0, max = 1)
}
\arguments{
\item{x}{value(s) to be transformed}

\item{min}{Lower end of logit interval}

\item{max}{Upper end of logit interval}
}
\value{
Transformed value(s).
}
\description{
Compute generalized logit and generalized inverse logit functions.
}
\details{
The generalized logit function takes values on [min, max] and transforms
them to span [-Inf,Inf] it is defined as:

\deqn{y = log(\frac{p}{(1-p)})}{y = log(p/(1-p))}

where

\deqn{p=\frac{(x-min)}{(max-min)}}{p=(x-min)/(max-min)}

The generalized inverse logit function provides the inverse transformation:

\deqn{x = p' (max-min) + min}{x = p * (max-min) + min}

where

\deqn{p'=\frac{exp(y)}{(1+exp(y))}}{exp(y)/(1+exp(y))}
}
\examples{


x <- seq(0, 10, by = 0.25)
xt <- logit(x, min = 0, max = 10)
cbind(x, xt)

y <- inv.logit(xt, min = 0, max = 10)
cbind(x, xt, y)
}
\seealso{
\code{\link[car]{logit}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{math}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smartbind.R
\name{smartbind}
\alias{smartbind}
\title{Efficient rbind of data frames, even if the column names don't match}
\usage{
smartbind(..., list, fill = NA, sep = ":", verbose = FALSE)
}
\arguments{
\item{\dots}{Data frames to combine}

\item{list}{List containing data frames to combine}

\item{fill}{Value to use when 'filling' missing columns.  Defaults to
\code{NA}.}

\item{sep}{Character string used to separate column names when pasting them
together.}

\item{verbose}{Logical flag indicating whether to display processing
messages. Defaults to \code{FALSE}.}
}
\value{
The returned data frame will contain: \item{columns}{all columns
present in any provided data frame} \item{rows}{a set of rows from each
provided data frame, with values in columns not present in the given data
frame filled with missing (\code{NA}) values.} The data type of columns will
be preserved, as long as all data frames with a given column name agree on
the data type of that column.  If the data frames disagree, the column will
be converted into a character strings.  The user will need to coerce such
character columns into an appropriate type.
}
\description{
Efficient rbind of data frames, even if the column names don't match
}
\examples{


df1 <- data.frame(A = 1:10, B = LETTERS[1:10], C = rnorm(10))
df2 <- data.frame(A = 11:20, D = rnorm(10), E = letters[1:10])

# rbind would fail
\dontrun{
rbind(df1, df2)
# Error in match.names(clabs, names(xi)) : names do not match previous
# names:
# 	D, E
}
# but smartbind combines them, appropriately creating NA entries
smartbind(df1, df2)

# specify fill=0 to put 0 into the missing row entries
smartbind(df1, df2, fill = 0)
\dontshow{
n <- 10 # number of data frames to create
s <- 10 # number of rows in each data frame

# create a bunch of column names
names <- LETTERS[2:5]

# create a list 'Z' containing 'n' data frames, each with 3 columns
# and 's' rows.  The first column is always named 'A', but the other
# two have a names randomly selected from 'names'

Z <- list()
for (i in 1:n)
{
  X <- data.frame(
    A = sample(letters, s, replace = TRUE),
    B = letters[1:s],
    C = rnorm(s)
  )
  colnames(X) <- c("A", sample(names, 2, replace = FALSE))
  Z[[i]] <- X
}

# Error in match.names(clabs, names(xi)) : names do not match
# previous names: E

# But smartbind will 'do the right thing'
df <- do.call("smartbind", Z)
df

# Equivalent call:
df <- smartbind(list = Z)
}

}
\seealso{
\code{\link{rbind}}, \code{\link{cbind}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lastAdd.R
\name{lastAdd}
\alias{lastAdd}
\title{Non-destructively construct a .Last function to be executed when R exits.}
\usage{
lastAdd(fun)
}
\arguments{
\item{fun}{Function to be called.}
}
\value{
A new function to be used for \code{.Last}.
}
\description{
Non-destructively construct a \code{.Last} function to be executed when R
exits.
}
\details{
\code{lastAdd} constructs a new function which can be used to replace the
existing definition of \code{.Last}, which will be executed when R terminates
normally.

If a \code{.Last} function already exists in the global environment, the
original definition is stored in a private environment, and the new function
is defined to call the function \code{fun} and then to call the previous
(stored) definition of \code{.Last}.

If no \code{.Last} function exists in the global environment, \code{lastAdd}
simply returns the function \code{fun}.
}
\note{
This function replaces the (now defunct) \code{addLast} function.
}
\examples{


## Print a couple of cute messages when R exits.
helloWorld <- function() cat("\nHello World!\n")
byeWorld <- function() cat("\nGoodbye World!\n")

.Last <- lastAdd(byeWorld)
.Last <- lastAdd(helloWorld)
\dontshow{
.Last()
}
\dontrun{
q("no")

## Should yield:
##
##   Save workspace image? [y/n/c]: n
##
##   Hello World!
##
##   Goodbye World!
##
##   Process R finished at Tue Nov 22 10:28:55 2005
}

}
\seealso{
\code{\link[base]{.Last}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gtools-package.R
\docType{package}
\name{gtools}
\alias{gtools}
\alias{gtools-package}
\title{gtools: Various R Programming Tools}
\description{
Functions to assist in R programming, including:
\describe{
\item{developing, updating, and maintaining R and R packages}{'ask', 'checkRVersion', 'getDependencies', 'keywords', 'scat'}
\item{calculate the logit and inverse logit transformations}{'logit', 'inv.logit'}
\item{test if a value is missing, empty, contains only NA and NULL values, or is a 'try-error'}{'invalid'}
\item{manipulate R's .Last function}{'addLast'}
\item{define macros}{'defmacro', 'strmacro'}
\item{detect odd and even integers}{'odd', 'even'}
\item{convert strings containing non-ASCII characters (like single quotes) to plain ASCII}{'ASCIIfy'}
\item{perform a binary search}{'binsearch'}
\item{sort strings containing both numeric and character components}{'mixedsort', 'mixedorder'}
\item{create a factor variable from the quantiles of a continuous variable}{'quantcut'}
\item{enumerate permutations and combinations}{'combinations', 'permutation'}
\item{calculate and convert between fold-change and log-ratio}{'foldchange', 'logratio2foldchange', 'foldchange2logratio'}
\item{calculate probabilities and generate random numbers from Dirichlet distributions}{'rdirichlet', 'ddirichlet'}
\item{apply a function over adjacent subsets of a vector}{'running'}
\item{modify the TCP_NODELAY ('de-Nagle') flag for socket objects}{'tcpNoDelay'}
\item{efficient 'rbind' of data frames, even if the column names don't match}{'smartbind'}
\item{generate significance stars from p-values}{'stars.pval'}
\item{convert characters to/from ASCII codes}{asc, chr}
\item{convert character vector to ASCII representation}{'ASCIIfy'}
\item{apply title capitalization rules to a character vector}{'capwords'}
}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/r-gregmisc/gtools}
  \item Report bugs at \url{https://github.com/r-gregmisc/gtools/issues}
}

}
\author{
\strong{Maintainer}: Ben Bolker \email{bolker@mcmaster.ca} (\href{https://orcid.org/0000-0002-2127-0443}{ORCID})

Authors:
\itemize{
  \item Gregory R. Warnes
  \item Thomas Lumley
  \item Arni Magnusson
  \item Bill Venables
  \item Genei Ryodan
  \item Steffen Moeller
}

Other contributors:
\itemize{
  \item Ian Wilson [contributor]
  \item Mark Davis [contributor]
  \item Nitin Jain [contributor]
  \item Scott Chamberlain [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unByteCode.R
\name{unByteCode}
\alias{unByteCode}
\alias{unByteCodeAssign}
\alias{assignEdgewise}
\title{Convert a byte-code function to an interpreted-code function}
\usage{
unByteCode(fun)

assignEdgewise(name, env, value)

unByteCodeAssign(fun)
}
\arguments{
\item{fun}{function to be modified}

\item{name}{object name}

\item{env}{namespace}

\item{value}{new function body}
}
\value{
All three functions return a copy of the modified function or
assigned value.
}
\description{
The purpose of these functions is to allow a byte coded function to be
converted back into a fully interpreted function as a \emph{temporary} work
around for issues in byte-code interpretation.
}
\details{
\code{unByteCode} returns a copy of the function that is directly
interpreted from text rather than from byte-code.

\code{assignEdgewise} makes an assignment into a locked environment.

\code{unByteCodeAssign} changes the specified function \emph{in its source
environment} to be directly interpreted from text rather than from
byte-code.

The latter two functions no longer work out of the box because \code{assignEdgewise}
(which \code{unByteCodeAssign} uses) makes use of an unsafe \code{unlockBinding}
call, but running \code{assignEdgewise()} will
}
\note{
These functions are not intended as a permanent solution to issues
with byte-code compilation or interpretation.  Any such issues should be
promptly reported to the R maintainers via the R Bug Tracking System at
\url{https://bugs.r-project.org} and via the R-devel mailing list
\url{https://stat.ethz.ch/mailman/listinfo/r-devel}.
}
\examples{

data(badDend)
dist2 <- function(x) as.dist(1 - cor(t(x), method = "pearson"))
hclust1 <- function(x) hclust(x, method = "single")

distance <- dist2(badDend)
cluster <- hclust1(distance)

dend <- as.dendrogram(cluster)
\dontrun{
## In R 2.3.0 and earlier crashes with a node stack overflow error
plot(dend)
## Error in xy.coords(x, y, recycle = TRUE) : node stack overflow
}

## convert stats:::plotNode from byte-code to interpreted-code
## (no longer available unless assignEdgewise is defined by the user)
## unByteCodeAssign(stats:::plotNode)
## illustrated in https://stackoverflow.com/questions/16559250/error-in-heatmap-2-gplots

# increase recursion limit
options("expressions" = 5e4)

# now the function does not crash
plot(dend)
}
\references{
These functions were inspired as a work-around to R bug
\url{https://bugs.r-project.org/show_bug.cgi?id=15215}.
}
\seealso{
\code{\link[compiler]{disassemble}}, \code{\link{assign}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{programming}
\keyword{utilites}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permute.R
\name{permute}
\alias{permute}
\title{Randomly Permute the Elements of a Vector}
\usage{
permute(x)
}
\arguments{
\item{x}{Vector of items to be permuted}
}
\value{
Vector with the original items reordered.
}
\description{
Randomly Permute the elements of a vector
}
\details{
This is simply a wrapper function for \code{\link{sample}}.
}
\examples{

x <- 1:10
permute(x)
}
\seealso{
\code{\link{sample}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ELISA.R
\docType{data}
\name{ELISA}
\alias{ELISA}
\title{Data from an ELISA assay}
\format{
a data frame with the following columns:
\itemize{
  \item PlateDay factor. Specifies one of four physically distinct 96 well plates
  \item Read factor. The signal was read 3 times for each plate.
  \item Description character. Indicates contents of sample.
  \item Concentration numeric. Nominal concentration of standards (NA for all other samples).
  \item Signal numeric. Assay signal. Specifically, optical density (a colorimetric assay).
}
}
\source{
Anonymized data.
}
\description{
Observed signals and (for some observations) nominal concentrations for
samples that were aliquoted to multiple assay plates, which were read
multiple times on multiple days.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quantcut.R
\name{quantcut}
\alias{quantcut}
\title{Create a Factor Variable Using the Quantiles of a Continuous Variable}
\usage{
quantcut(x, q = 4, na.rm = TRUE, ...)
}
\arguments{
\item{x}{Continuous variable.}

\item{q}{Either a integer number of equally spaced quantile groups to
create, or a vector of quantiles used for creating groups. Defaults to
\code{q=4} which is equivalent to \code{q=seq(0, 1, by=0.25)}.  See
\code{\link{quantile}} for details.}

\item{na.rm}{Boolean indicating whether missing values should be removed
when computing quantiles.  Defaults to TRUE.}

\item{\dots}{Optional arguments passed to \code{\link{cut}}.}
}
\value{
Factor variable with one level for each quantile interval.
}
\description{
Create a factor variable using the quantiles of a continuous variable.
}
\details{
This function uses \code{\link{quantile}} to obtain the specified quantiles
of \code{x}, then calls \code{\link{cut}} to create a factor variable using
the intervals specified by these quantiles.

It properly handles cases where more than one quantile obtains the same
value, as in the second example below.  Note that in this case, there will
be fewer generated factor levels than the specified number of quantile
intervals.
}
\examples{


## create example data
# testonly{
set.seed(1234)
# }
x <- rnorm(1000)

## cut into quartiles
quartiles <- quantcut(x)
table(quartiles)

## cut into deciles
deciles.1 <- quantcut(x, 10)
table(deciles.1)
# or equivalently
deciles.2 <- quantcut(x, seq(0, 1, by = 0.1))

# testonly{
stopifnot(identical(deciles.1, deciles.2))
# }

## show handling of 'tied' quantiles.
x <- round(x) # discretize to create ties
stem(x) # display the ties
deciles <- quantcut(x, 10)

table(deciles) # note that there are only 5 groups (not 10)
# due to duplicates
}
\seealso{
\code{\link{cut}}, \code{\link{quantile}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ASCIIfy.R
\name{ASCIIfy}
\alias{ASCIIfy}
\title{Convert Characters to ASCII}
\usage{
ASCIIfy(x, bytes = 2, fallback = "?")
}
\arguments{
\item{x}{a character vector, possibly containing non-ASCII characters.}

\item{bytes}{either \code{1} or \code{2}, for single-byte (\samp{\x00}) or
two-byte (\samp{\u0000}) codes.}

\item{fallback}{an output character to use, when input characters cannot be
converted.}
}
\value{
A character vector like \code{x}, except non-ASCII characters have been
replaced with \samp{\x00} or \samp{\u0000} codes.
}
\description{
Convert character vector to ASCII, replacing non-ASCII characters with
single-byte (\samp{\x00}) or two-byte (\samp{\u0000}) codes.
}
\note{
To render single backslashes, use these or similar techniques: \preformatted{
write(ASCIIfy(x), "file.txt")
cat(paste(ASCIIfy(x), collapse="\n"), "\n", sep="")}

The resulting strings are plain ASCII and can be used in R functions and
datasets to improve package portability.
}
\examples{
cities <- c("S\u00e3o Paulo", "Reykjav\u00edk")
print(cities)
ASCIIfy(cities, 1)
ASCIIfy(cities, 2)

athens <- "\u0391\u03b8\u03ae\u03bd\u03b1"
print(athens)
ASCIIfy(athens)

}
\seealso{
\code{\link[tools]{showNonASCII}} identifies non-ASCII characters in a
character vector.
}
\author{
Arni Magnusson.
}
\keyword{character}
\keyword{utilites}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/na.replace.R
\name{na.replace}
\alias{na.replace}
\title{Replace Missing Values}
\usage{
na.replace(x, replace, ...)
}
\arguments{
\item{x}{vector possibly containing missing (\code{NA}) values}

\item{replace}{either a scalar replacement value, or a function returning a
scalar value}

\item{...}{Optional arguments to be passed to \code{replace}}
}
\value{
Vector with missing values (\code{NA}) replaced by the value of
\code{replace}.
}
\description{
Replace missing values
}
\details{
This is a convenience function that is the same as x[is.na(x)] <- replace
}
\examples{

x <- c(1, 2, 3, NA, 6, 7, 8, NA, NA)

# Replace with a specified value
na.replace(x, "999")

# Replace with the calculated median
na.replace(x, median, na.rm = TRUE)
}
\seealso{
\code{\link[base]{is.na}}, \code{\link[stats]{na.omit}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixedsort.R
\name{mixedsort}
\alias{mixedsort}
\alias{mixedorder}
\title{Order or Sort strings with embedded numbers so that the numbers are in the
correct order}
\usage{
mixedsort(
  x,
  decreasing = FALSE,
  na.last = TRUE,
  blank.last = FALSE,
  numeric.type = c("decimal", "roman"),
  roman.case = c("upper", "lower", "both"),
  scientific = TRUE
)

mixedorder(
  x,
  decreasing = FALSE,
  na.last = TRUE,
  blank.last = FALSE,
  numeric.type = c("decimal", "roman"),
  roman.case = c("upper", "lower", "both"),
  scientific = TRUE
)
}
\arguments{
\item{x}{Vector to be sorted.}

\item{decreasing}{logical.  Should the sort be increasing or decreasing?
Note that \code{descending=TRUE} reverses the meanings of \code{na.last} and
\code{blanks.last}.}

\item{na.last}{for controlling the treatment of \code{NA} values.  If
\code{TRUE}, missing values in the data are put last; if \code{FALSE}, they
are put first; if \code{NA}, they are removed.}

\item{blank.last}{for controlling the treatment of blank values.  If
\code{TRUE}, blank values in the data are put last; if \code{FALSE}, they
are put first; if \code{NA}, they are removed.}

\item{numeric.type}{either "decimal" (default) or "roman".  Are numeric
values represented as decimal numbers (\code{numeric.type="decimal"}) or as
Roman numerals (\code{numeric.type="roman"})?}

\item{roman.case}{one of "upper", "lower", or "both".  Are roman numerals
represented using only capital letters ('IX') or lower-case letters ('ix')
or both?}

\item{scientific}{logical. Should exponential notation be allowed for numeric values.}
}
\value{
\code{mixedorder} returns a vector giving the sort order of the
input elements. \code{mixedsort} returns the sorted vector.
}
\description{
These functions sort or order character strings containing embedded numbers
so that the numbers are numerically sorted rather than sorted by character
value.  I.e. "Aspirin 50mg" will come before "Aspirin 100mg".  In addition,
case of character strings is ignored so that "a", will come before "B" and
"C".
}
\details{
I often have character vectors (e.g. factor labels), such as compound and
dose, that contain both text and numeric data.  This function is useful for
sorting these character vectors into a logical order.

It does so by splitting each character vector into a sequence of character
and numeric sections, and then sorting along these sections, with numbers
being sorted by numeric value (e.g. "50" comes before "100"), followed by
characters strings sorted by character value (e.g. "A" comes before "B")
\emph{ignoring case} (e.g. 'A' has the same sort order as 'a').

By default, sort order is ascending, empty strings are sorted to the front,
and \code{NA} values to the end.  Setting \code{descending=TRUE} changes the
sort order to descending and reverses the meanings of \code{na.last} and
\code{blank.last}.

Parsing looks for decimal numbers unless \code{numeric.type="roman"}, in
which parsing looks for roman numerals, with character case specified by
\code{roman.case}.
}
\examples{

## compound & dose labels
Treatment <- c(
  "Control", "Aspirin 10mg/day", "Aspirin 50mg/day",
  "Aspirin 100mg/day", "Acetomycin 100mg/day",
  "Acetomycin 1000mg/day"
)

## ordinary sort puts the dosages in the wrong order
sort(Treatment)

## but mixedsort does the 'right' thing
mixedsort(Treatment)

## Here is a more complex example
x <- rev(c(
  "AA 0.50 ml", "AA 1.5 ml", "AA 500 ml", "AA 1500 ml",
  "EXP 1", "AA 1e3 ml", "A A A", "1 2 3 A", "NA", NA, "1e2",
  "", "-", "1A", "1 A", "100", "100A", "Inf"
))

mixedorder(x)

mixedsort(x) # Notice that plain numbers, including 'Inf' show up
# before strings, NAs at the end, and blanks at the
# beginning .


mixedsort(x, na.last = TRUE) # default
mixedsort(x, na.last = FALSE) # push NAs to the front


mixedsort(x, blank.last = FALSE) # default
mixedsort(x, blank.last = TRUE) # push blanks to the end

mixedsort(x, decreasing = FALSE) # default
mixedsort(x, decreasing = TRUE) # reverse sort order

## Roman numerals
chapters <- c(
  "V. Non Sequiturs", "II. More Nonsense",
  "I. Nonsense", "IV. Nonesensical Citations",
  "III. Utter Nonsense"
)
mixedsort(chapters, numeric.type = "roman")

## Lower-case Roman numerals
vals <- c(
  "xix", "xii", "mcv", "iii", "iv", "dcclxxii", "cdxcii",
  "dcxcviii", "dcvi", "cci"
)
(ordered <- mixedsort(vals, numeric.type = "roman", roman.case = "lower"))
roman2int(ordered)

## Control scientific notation for number matching:
vals <- c("3E1", "2E3", "4e0")

mixedsort(vals) # With scientfic notation
mixedsort(vals, scientific = FALSE) # Without scientfic notation
}
\seealso{
\code{\link[base]{sort}}, \code{\link[base]{order}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{manip}
\keyword{univar}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gtools-defunct.R
\name{assert}
\alias{assert}
\alias{gtools-defunct}
\alias{addLast}
\alias{capture}
\alias{sprint}
\title{Defunct Functions in package \code{gtools}}
\usage{
assert(...)

capture(expression, collapse = "\\n")

sprint(x, ...)
}
\arguments{
\item{expression, collapse, x, \dots}{ignored}
}
\description{
The functions or variables listed here are no longer part of package
\code{gtools}.
}
\details{
\itemize{
\item \code{assert} is a defunct synonym for \code{\link[base]{stopifnot}}.
\item \code{addLast} has been replaced by \code{lastAdd}, which has the same
  purpose but applied using different syntax.
\item \code{capture} and \code{capture.output} have been removed in favor of
  \code{capture.output} from the \code{utils} package.
}
}
\seealso{
\code{\link[base]{Defunct}},
  \code{\link[base]{stopifnot}},
  \code{\link[gtools]{lastAdd}},
  \code{\link[utils]{capture.output}}
}
\keyword{misc}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oddeven.R
\name{oddeven}
\alias{oddeven}
\alias{odd}
\alias{even}
\title{Detect odd/even integers}
\usage{
odd(x)

even(x)
}
\arguments{
\item{x}{vector of integers}
}
\value{
Vector of TRUE/FALSE values.
}
\description{
detect odd/even integers
}
\examples{


odd(4)
even(4)

odd(1:10)
even(1:10)
}
\seealso{
\code{\link[base]{round}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{arith}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stars.pval.R
\name{stars.pval}
\alias{stars.pval}
\title{Generate significance stars from p-values}
\usage{
stars.pval(p.value)
}
\arguments{
\item{p.value}{numeric vector of p-values}
}
\value{
A character vector containing the same number of elements as
\code{p-value}, with an attribute "legend" providing the conversion pattern.
}
\description{
Generate significance stars (e.g. '***', '**', '*', '.') from p-values using
R's standard definitions.
}
\details{
Mapping from p-value ranges to symbols: \describe{ \item{0 - 0.001}{'***'}
\item{0.001 - 0.01}{'**'} \item{0.01 - 0.05}{'*'} \item{0.05 - 0.1}{'.'}
\item{0.1 - 1.0}{'' (No symbol)} }
}
\examples{

p.val <- c(0.0004, 0.0015, 0.013, 0.044, 0.067, 0.24)
stars.pval(p.val)
}
\seealso{
\code{\link[stats]{symnum}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{misc}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/keywords.R
\name{keywords}
\alias{keywords}
\title{List valid keywords for R man pages}
\usage{
keywords(topic)
}
\arguments{
\item{topic}{object or man page topic}
}
\description{
List valid keywords for R man pages
}
\details{
If \code{topic} is provided, return a list of the keywords associated with
\code{topic}.  Otherwise, display the list of valid R keywords from the R
doc/KEYWORDS file.
}
\examples{

## Show all valid R keywords
\dontrun{
keywords()

## Show keywords associated with the 'merge' function
keywords(merge)
keywords("merge")
}
}
\seealso{
\code{\link[utils]{help}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{documentation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defmacro.R, R/strmacro.R
\name{defmacro}
\alias{defmacro}
\alias{strmacro}
\title{Define a macro}
\usage{
defmacro(..., expr)

strmacro(..., expr, strexpr)
}
\arguments{
\item{\dots}{macro argument list}

\item{expr}{R expression defining the macro body}

\item{strexpr}{character string defining the macro body}
}
\value{
A macro function.
}
\description{
\code{defmacro} define a macro that uses R expression replacement
}
\details{
\code{strmacro} define a macro that uses string replacement


\code{defmacro} and \code{strmacro} create a macro from the expression given
in \code{expr}, with formal arguments given by the other elements of the
argument list.

A macro is similar to a function definition except for handling of formal
arguments.  In a function, formal arguments are simply variables that
contains the result of evaluating the expressions provided to the function
call.  In contrast, macros actually modify the macro body by
\emph{replacing} each formal argument by the expression (\code{defmacro}) or
string (\code{strmacro}) provided to the macro call.

For \code{defmacro}, the special argument name \code{DOTS} will be replaced
by \code{...} in the formal argument list of the macro so that \code{...} in
the body of the expression can be used to obtain any additional arguments
passed to the macro. For \code{strmacro} you can mimic this behavior
providing a \code{DOTS=""} argument.  This is illustrated by the last
example below.

Macros are often useful for creating new functions during code execution.
}
\note{
Note that because [the defmacro code] works on the parsed expression,
not on a text string, defmacro avoids some of the problems of traditional
string substitution macros such as \code{strmacro} and the C preprocessor
macros. For example, in \preformatted{ mul <- defmacro(a, b, expr={a*b}) } a
C programmer might expect \code{mul(i, j + k)} to expand (incorrectly) to
\code{i*j + k}. In fact it expands correctly, to the equivalent of
\code{i*(j + k)}.

For a discussion of the differences between functions and macros, please
Thomas Lumley's R-News article (reference below).
}
\examples{

####
# macro for replacing a specified missing value indicator with NA
# within a dataframe
###
setNA <- defmacro(df, var, values,
  expr = {
    df$var[df$var \%in\% values] <- NA
  }
)

# create example data using 999 as a missing value indicator
d <- data.frame(
  Grp = c("Trt", "Ctl", "Ctl", "Trt", "Ctl", "Ctl", "Trt", "Ctl", "Trt", "Ctl"),
  V1 = c(1, 2, 3, 4, 5, 6, 999, 8, 9, 10),
  V2 = c(1, 1, 1, 1, 1, 2, 999, 2, 999, 999),
  stringsAsFactors = TRUE
)
d

# Try it out
setNA(d, V1, 999)
setNA(d, V2, 999)
d


###
# Expression macro
###
plot.d <- defmacro(df, var, DOTS,
  col = "red", title = "", expr =
    plot(df$var ~ df$Grp, type = "b", col = col, main = title, ...)
)

plot.d(d, V1)
plot.d(d, V1, col = "blue")
plot.d(d, V1, lwd = 4) # use optional 'DOTS' argument

###
# String macro (note the quoted text in the calls below)
#
# This style of macro can be useful when you are reading
# function arguments from a text file
###
plot.s <- strmacro(DF, VAR,
  COL = "'red'", TITLE = "''", DOTS = "", expr =
    plot(DF$VAR ~ DF$Grp, type = "b", col = COL, main = TITLE, DOTS)
)

plot.s("d", "V1")
plot.s(DF = "d", VAR = "V1", COL = '"blue"')
plot.s("d", "V1", DOTS = "lwd=4") # use optional 'DOTS' argument



#######
# Create a macro that defines new functions
######
plot.sf <- defmacro(
  type = "b", col = "black",
  title = deparse(substitute(x)), DOTS, expr =
    function(x, y) plot(x, y, type = type, col = col, main = title, ...)
)

plot.red <- plot.sf(col = "red", title = "Red is more Fun!")
plot.blue <- plot.sf(col = "blue", title = "Blue is Best!", lty = 2)

plot.red(1:100, rnorm(100))
plot.blue(1:100, rnorm(100))
}
\references{
The original \code{defmacro} code was directly taken from:

Lumley T. "Programmer's Niche: Macros in R", R News, 2001, Vol 1, No. 3, pp
11--13, \url{https://cran.r-project.org/doc/Rnews/}
}
\seealso{
\code{\link[base]{function}} \code{\link[base]{substitute}},
\code{\link[base]{eval}}, \code{\link[base]{parse}},
\code{\link[base]{source}}, \code{\link[base]{parse}},
}
\author{
Thomas Lumley wrote \code{defmacro}.  Gregory R. Warnes
\email{greg@warnes.net} enhanced it and created \code{strmacro}.
}
\keyword{##}
\keyword{Code}
\keyword{from}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roman2int.R
\name{roman2int}
\alias{roman2int}
\title{Convert Roman Numerals to Integers}
\usage{
roman2int(roman)
}
\arguments{
\item{roman}{character vector containing roman numerals}
}
\value{
A integer vector with the same length as \code{roman}.  Character
strings which are not valid roman numerals will be converted to \code{NA}.
}
\description{
Convert roman numerals to integers
}
\details{
This function will convert roman numerals to integers without the upper bound
imposed by R (3899), ignoring case.
}
\examples{

roman2int(c("I", "V", "X", "C", "L", "D", "M"))

# works regardless of case
roman2int("MMXVI")
roman2int("mmxvi")

# works beyond R's limit of 3899
val.3899 <- "MMMDCCCXCIX"
val.3900 <- "MMMCM"
val.4000 <- "MMMM"
as.numeric(as.roman(val.3899))
as.numeric(as.roman(val.3900))
as.numeric(as.roman(val.4000))

roman2int(val.3899)
roman2int(val.3900)
roman2int(val.4000)
}
\seealso{
\code{\link[utils]{as.roman}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}
}
\keyword{arith}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDependencies.R
\name{getDependencies}
\alias{getDependencies}
\title{Get package dependencies}
\usage{
getDependencies(
  pkgs,
  dependencies = c("Depends", "Imports", "LinkingTo"),
  installed = TRUE,
  available = TRUE,
  base = FALSE,
  recommended = FALSE
)
}
\arguments{
\item{pkgs}{character vector of package names}

\item{dependencies}{character vector of dependency types to include.
Choices are "Depends", "Imports", "LinkingTo", "Suggests", and "Enhances".
Defaults to \code{c("Depends", "Imports", "LinkingTo")}.}

\item{installed}{Logical indicating whether to pull dependency information
from installed packages. Defaults to TRUE.}

\item{available}{Logical indicating whether to pull dependency information
from available packages.  Defaults to TRUE.}

\item{base}{Logical indicating whether to include dependencies on base
packages that are included in the R installation. Defaults to FALSE.}

\item{recommended}{Logical indicating whether to include dependencies on
recommended packages that are included in the R installation. Defaults to
FALSE.}
}
\value{
A character vector of package names.
}
\description{
Get package dependencies
}
\details{
This function recursively constructs the list of dependencies for the
packages given by \code{pkgs}.  By default, the dependency information is
extracted from both installed and available packages.  As a consequence, it
works both for local and CRAN packages.
}
\note{
If \code{available=TRUE} R will attempt to access the currently
selected CRAN repository, prompting for one if necessary.
}
\examples{

\dontrun{
## A locally installed package
#' getDependencies("MASS", installed = TRUE, available = FALSE)

## A package on CRAN
getDependencies("gregmisc", installed = FALSE, available = TRUE)


## Show base and recommended dependencies
getDependencies("MASS", available = FALSE, base = TRUE, recommended = TRUE)

## Download the set of packages necessary to support a local package
deps <- getDependencies("MyLocalPackage", available = FALSE)
download.packages(deps, destdir = "./R_Packages")
}

}
\seealso{
\code{\link{installed.packages}}, \code{\link{available.packages}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net} based on the non exported
\code{utils:::getDependencies} and \code{utils:::.clean_up_dependencies2}.
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/running.R
\name{running}
\alias{running}
\title{Apply a Function Over Adjacent Subsets of a Vector}
\usage{
running(
  X,
  Y = NULL,
  fun = mean,
  width = min(length(X), 20),
  allow.fewer = FALSE,
  pad = FALSE,
  align = c("right", "center", "left"),
  simplify = TRUE,
  by,
  ...
)
}
\arguments{
\item{X}{data vector}

\item{Y}{data vector (optional)}

\item{fun}{Function to apply. Default is \code{mean}}

\item{width}{Integer giving the number of vector elements to include in the
subsets.  Defaults to the lesser of the length of the data and 20 elements.}

\item{allow.fewer}{Boolean indicating whether the function should be
computed for subsets with fewer than \code{width} points}

\item{pad}{Boolean indicating whether the returned results should be
'padded' with NAs corresponding to sets with less than \code{width}
elements.  This only applies when when \code{allow.fewer} is FALSE.}

\item{align}{One of "right", "center", or "left".  This controls the
relative location of `short' subsets with less then \code{width} elements:
"right" allows short subsets only at the beginning of the sequence so that
all of the complete subsets are at the end of the sequence (i.e. `right
aligned'), "left" allows short subsets only at the end of the data so that
the complete subsets are `left aligned', and "center" allows short subsets
at both ends of the data so that complete subsets are `centered'.}

\item{simplify}{Boolean.  If FALSE the returned object will be a list
containing one element per evaluation.  If TRUE, the returned object will be
coerced into a vector (if the computation returns a scalar) or a matrix (if
the computation returns multiple values).  Defaults to FALSE.}

\item{by}{Integer separation between groups. If \code{by=width} will give
non-overlapping windows. Default is missing, in which case groups will start
at each value in the X/Y range.}

\item{\dots}{parameters to be passed to \code{fun}}
}
\value{
List (if \code{simplify==TRUE}), vector, or matrix containing the
results of applying the function \code{fun} to the subsets of \code{X}
(\code{running}) or \code{X} and \code{Y}.

Note that this function will create a vector or matrix even for objects
which are not simplified by \code{sapply}.
}
\description{
Applies a function over subsets of the vector(s) formed by taking a fixed
number of previous points.
}
\details{
\code{running} applies the specified function to a sequential windows on
\code{X} and (optionally) \code{Y}.  If \code{Y} is specified the function
must be bivariate.
}
\examples{


# show effect of pad
running(1:20, width = 5)
running(1:20, width = 5, pad = TRUE)

# show effect of align
running(1:20, width = 5, align = "left", pad = TRUE)
running(1:20, width = 5, align = "center", pad = TRUE)
running(1:20, width = 5, align = "right", pad = TRUE)

# show effect of simplify
running(1:20, width = 5, fun = function(x) x) # matrix
running(1:20, width = 5, fun = function(x) x, simplify = FALSE) # list

# show effect of by
running(1:20, width = 5) # normal
running(1:20, width = 5, by = 5) # non-overlapping
running(1:20, width = 5, by = 2) # starting every 2nd


# Use 'pad' to ensure correct length of vector, also show the effect
# of allow.fewer.
par(mfrow = c(2, 1))
plot(1:20, running(1:20, width = 5, allow.fewer = FALSE, pad = TRUE), type = "b")
plot(1:20, running(1:20, width = 5, allow.fewer = TRUE, pad = TRUE), type = "b")
par(mfrow = c(1, 1))

# plot running mean and central 2 standard deviation range
# estimated by *last* 40 observations
dat <- rnorm(500, sd = 1 + (1:500) / 500)
plot(dat)
sdfun <- function(x, sign = 1) mean(x) + sign * sqrt(var(x))
lines(running(dat, width = 51, pad = TRUE, fun = mean), col = "blue")
lines(running(dat, width = 51, pad = TRUE, fun = sdfun, sign = -1), col = "red")
lines(running(dat, width = 51, pad = TRUE, fun = sdfun, sign = 1), col = "red")


# plot running correlation estimated by last 40 observations (red)
# against the true local correlation (blue)
sd.Y <- seq(0, 1, length = 500)

X <- rnorm(500, sd = 1)
Y <- rnorm(500, sd = sd.Y)

plot(running(X, X + Y, width = 20, fun = cor, pad = TRUE), col = "red", type = "s")

r <- 1 / sqrt(1 + sd.Y^2) # true cor of (X,X+Y)
lines(r, type = "l", col = "blue")
}
\seealso{
\code{\link[gplots]{wapply}} to apply a function over an x-y window
centered at each x point, \code{\link[base]{sapply}},
\code{\link[base]{lapply}}
}
\author{
Gregory R. Warnes \email{greg@warnes.net}, with contributions by
Nitin Jain \email{nitin.jain@pfizer.com}.
}
\keyword{misc}
